package hello.jsp;

import javax.servlet.*;
import javax.servlet.http.*; 
import java.io.*;

import org.apache.cactus.*;
import junit.framework.*;
import org.apache.struts.action.*;
import org.apache.struts.config.ForwardConfig;

/**
 * Tests the XML implementation only.
 * $Revision: 1.2 $ $Date: 2002/10/06 22:46:48 $ (helder@acm.org).
 */
public class ShowMessagesActionTest extends ServletTestCase {
    
    public ShowMessagesActionTest(String name) {
        super(name);
    }
    public static Test suite() {
        return new TestSuite(ShowMessagesActionTest.class);
    }

    private ShowMessagesAction command;
    private String testMessage = "This is a test message"; // this is the text for all messages
    private ActionMapping mapping;
    private NewMessageForm form; 
    private MessageBeanDAO testDAO;
    
    public void beginExecute(WebRequest clientRequest) {
        // no parameters to set
    }
    
    public void setUp() throws IOException {
        testDAO = new XMLMessageBeanDAO("/tmp/test");
        request.setAttribute("dao", testDAO);
        mapping = new ActionMapping();
        mapping.addForwardConfig(new ActionForward("failure", "/index.jsp", false)); // global definition
        mapping.addForwardConfig(new ActionForward("success", "/messages.jsp", false));
        form = new NewMessageForm();
        command = new ShowMessagesAction();
    }
    
    /**
     * This test is practically the same as the test for NewMessageCommand. The difference is
     * that the message received was unmarshaled from the XML file. Additionally, this test
     * will only run if the files exist (ex: the other test must run before this one.)
     */
    public void testExecute() throws ServletException, IOException {
        ActionForward fwdObj = command.execute(mapping, form, request, response);
        
        // dao attribute should exist
        MessageBeanDAO dao = (MessageBeanDAO)request.getAttribute("dao");
        assertNotNull("Attribute not set.", dao);
        assertEquals("Wrong data locator", "/tmp/test", dao.getLocator());
        
        // message.count should exist
        File countFile = new File("/tmp/test/message.count");
        assertTrue("Count file does not exist!", countFile.exists());
        
        // lastMessage attribute in request should exist with array of messageBean
        Object attribute = request.getAttribute("messages");
        assertNotNull("No attribute messages exists in Request.", attribute);
        MessageBean[] beans = (MessageBean[])attribute;
        
        // beans.length should be equivalent to the number of message_?.xml files in messageDir
        int files = this.getTotalFiles();
        assertEquals("Array lengths don't match", files, beans.length);
        
        // Open each bean and assert that each message is not null
        for (int i = 0; i < beans.length; i++) {
            String result = beans[i].getMessage();
            assertNotNull("Message is null.", result);
            System.out.println("Message found: " + result);
        }
        
        // returned object should contain success argument
         // This asserts that the forward name is what we expect - runtime value
        String target = fwdObj.getName();
        assertEquals("Unexpected returned target.", "success", target);
        
        // the value defined in struts-config.xml - configuration value
        String path = fwdObj.getPath();
        assertEquals("Unexpected returned path.", "/messages.jsp", path);
    }
    
    /**
     * Length of files in test directory
     */
    private int getTotalFiles() {
        String messageDir = "/tmp/test";
        File dir = new File(messageDir);
        File[] files = dir.listFiles(new FilenameFilter() {
            public boolean accept(File dir, String fileName) {
                return (new File(dir, fileName)).isFile() 
                       && fileName.endsWith(".xml")     
                       && fileName.startsWith("message_");
            }
        });
        return files.length;
    }
    
}
