package hello.rmi;

import java.rmi.*;

import hello.*;

public class HelloClient {
    
    public static void main (String[] args) throws java.net.MalformedURLException {

        if (System.getSecurityManager() == null) {
            System.setSecurityManager(new RMISecurityManager());
        }

        try {
            // Lookup "hellormi" object and cast to Remote interface type
            HelloRemote hello = (HelloRemote) Naming.lookup("hellormi");
        
            // We now have a local instance, and can call its methods
            // Print HelloRemote (call toString())
            System.out.println("Remote object: " + hello);
            
            System.out.println("\nResult of hello.sayHello(): " + hello.sayHello());
            System.out.println("Result of hello.getMessage().getNome(): " + hello.getMessage().getNome());
            System.out.println("Result of hello.getCirculo(): " + hello.getCirculo());

            System.out.println("----- Nomes e Strings -----");
            
            System.out.println("\nCalling hello.sayThis(\"Goodbye, Helder!\"): ");
            hello.sayThis("Goodbye, Helder!");
            System.out.println("Result of hello.sayHello(): " + hello.sayHello());
            System.out.println("Result of hello.getMessage().getNome(): " + hello.getMessage().getNome());

            System.out.println("\nCalling hello.getMessage().setNome(\"Hi, you guys!\"): ");
            hello.getMessage().setNome("Hi, you guys!");
            System.out.println("Result of hello.sayHello(): " + hello.sayHello());
            System.out.println("Result of hello.getMessage().getNome(): " + hello.getMessage().getNome());
            
            System.out.println("\nCalling hello.setMessage(hello.createNome(\"Bye, bye!\")): ");
            hello.setMessage(hello.createNome("Bye, bye!"));
            System.out.println("Result of hello.sayHello(): " + hello.sayHello());
            System.out.println("Result of hello.getMessage().getNome(): " + hello.getMessage().getNome());

            System.out.println("----- Circulos e Pontos -----");
            
            System.out.println("\nCalling hello.setCirculo(hello.createCirculo(50, hello.createPonto(30, 40)): ");
            hello.setCirculo(hello.createCirculo(50, hello.createPonto(30, 40)));
            System.out.println("Result of hello.getCirculo(): " + hello.getCirculo());
            System.out.println("Result of hello.getCirculo().getOrigem(): " + hello.getCirculo().getOrigem());

            System.out.println("\nCalling hello.getCirculo().setOrigem(new hello.createPonto(300,400)): ");
            hello.getCirculo().setOrigem(hello.createPonto(300,400));
            System.out.println("Result of hello.getCirculo(): " + hello.getCirculo());
            System.out.println("Result of hello.getCirculo().getOrigem(): " + hello.getCirculo().getOrigem());
            
            System.out.println("\nCalling hello.getCirculo().getOrigem().setX(3000): ");
            hello.getCirculo().getOrigem().setX(3000);
            System.out.println("Result of hello.getCirculo(): " + hello.getCirculo());
            System.out.println("Result of hello.getCirculo().getOrigem(): " + hello.getCirculo().getOrigem());
            
            
            // Lookup "hellormi" object and cast to Remote interface type
            
            
            System.out.println("----- Agora usando parametros que sao objetos remotos -----");
            HelloRpRemote helloRp = (HelloRpRemote) Naming.lookup("hellormi_rp");
        
            // We now have a local instance, and can call its methods
            // Print HelloRemote (call toString())
            System.out.println("Remote object: " + helloRp);
                        
            System.out.println("\nResult of helloRp.sayHello(): " + helloRp.sayHello());
            System.out.println("Result of helloRp.getMessage().getNome(): " + helloRp.getMessage().getNome());
            System.out.println("Result of helloRp.getCirculo(): " + helloRp.getCirculo().toStringRemote());

            System.out.println("----- Nomes e Strings -----");
            
            System.out.println("\nCalling helloRp.sayThis(\"Goodbye, Helder!\"): ");
            helloRp.sayThis("Goodbye, Helder!");
            System.out.println("Result of helloRp.sayHello(): " + helloRp.sayHello());
            System.out.println("Result of helloRp.getMessage().getNome(): " + helloRp.getMessage().getNome());

            System.out.println("\nCalling helloRp.getMessage().setNome(\"Hi, you guys!\"): ");
            helloRp.getMessage().setNome("Hi, you guys!");
            System.out.println("Result of helloRp.sayHello(): " + helloRp.sayHello());
            System.out.println("Result of helloRp.getMessage().getNome(): " + helloRp.getMessage().getNome());
            
            System.out.println("\nCalling helloRp.setMessage(helloRp.createNome(\"Bye, bye!\")): ");
            helloRp.setMessage(helloRp.createNome("Bye, bye!"));
            System.out.println("Result of helloRp.sayHello(): " + helloRp.sayHello());
            System.out.println("Result of helloRp.getMessage().getNome(): " + helloRp.getMessage().getNome());

            System.out.println("----- Circulos e Pontos -----");
            
            System.out.println("\nCalling helloRp.createCirculo(50, helloRp.createPonto(30, 40)): ");
            helloRp.setCirculo(helloRp.createCirculo(50, helloRp.createPonto(30, 40)));
            System.out.println("Result of helloRp.getCirculo(): " + helloRp.getCirculo().toStringRemote());
            System.out.println("Result of helloRp.getCirculo().getOrigem(): " + helloRp.getCirculo().getOrigem().toStringRemote());

            System.out.println("\nCalling helloRp.getCirculo().setOrigem(helloRp.createPonto(300,400)): ");
            helloRp.getCirculo().setOrigem(helloRp.createPonto(300,400));
            System.out.println("Result of helloRp.getCirculo(): " + helloRp.getCirculo().toStringRemote());
            System.out.println("Result of helloRp.getCirculo().getOrigem(): " + helloRp.getCirculo().getOrigem().toStringRemote());
            
            System.out.println("\nCalling helloRp.getCirculo().getOrigem().setX(3000): ");
            helloRp.getCirculo().getOrigem().setX(3000);
            System.out.println("Result of helloRp.getCirculo(): " + helloRp.getCirculo().toStringRemote());
            System.out.println("Result of helloRp.getCirculo().getOrigem(): " + helloRp.getCirculo().getOrigem().toStringRemote());

            
        } catch (RemoteException e) {
            System.out.println("" + e);
            e.printStackTrace();
        } catch (NotBoundException e) {
            System.out.println("No object is bound to 'hellormi'. " + e);
            e.printStackTrace();
        }
    }
    
}
