package hello.rmiop;

import java.rmi.*;
import javax.naming.*;

import hello.*;
import hello.portable.*; // RMI IIOP Remote Object Implementations

public class HelloServer {
    
    public static void main (String[] args) throws java.net.MalformedURLException {
        
        // Security Manager: requires policy file - see cap03/build.xml and cap03/lib
        if (System.getSecurityManager() == null) {
            System.setSecurityManager(new RMISecurityManager());
        }

        try {
            // Create remote object that uses local object parameters in methods
            // Create stored objects
            Circulo c = new Circulo(5, new Ponto(3, 4));
            Nome n = new Nome("Hello, World!");
            
            HelloRemote hello = new HelloImpl(n, c);
            
            // Create remote object that uses remote object parameters in methods
            // Create remote parameter objects
            CirculoRemote cr = new CirculoImpl(5, new PontoImpl(3, 4));
            NomeRemote nr = new NomeImpl("Hello, World!");
            
            HelloRpRemote helloRp = new HelloRpImpl(nr, cr);
        
            // Bind objects to JNDI Registry
            Context initCtx = new InitialContext(System.getProperties());
            
            initCtx.rebind("hellormiop", hello);
            System.out.println("Remote object bound to 'hellormiop'.");
            initCtx.rebind("hellormiop_rp", helloRp);
            System.out.println("Remote object bound to 'hellormiop_rp'. Server ready to receive requests.");

        } catch (RemoteException e) {
            System.out.println("" + e);
            e.printStackTrace();
        } catch (NamingException e) {
            System.out.println("" + e);
            e.printStackTrace();
        }
    }
    
}
