package loja;

import java.math.BigDecimal;
import java.rmi.*;

public class ProdutoImpl extends javax.rmi.PortableRemoteObject
                       			 implements Produto {
	
	private String codigo;
	private String nome;
	private BigDecimal preco;
	private int quantidade;
	
	ProdutoDAO dao;

	/**
	 * Cria novo produto com codigo, nome, preco e quantidade em estoque.
	 */
	public ProdutoImpl(String codigo, String nome, BigDecimal preco, int quantidade)  throws RemoteException {
		System.out.println("ProdutoImpl("+codigo+","+nome+","+preco+","+quantidade+")");
		this.codigo = codigo;
		this.nome = nome;
		this.preco = preco;
		this.quantidade = quantidade;
		
		try {
			dao = new ProdutoDAO();
		} catch (DataAccessException e) {
			throw new RemoteException("Falha ao inicializar o DAO: " + e);
		}
		
		// cria o objeto (no banco) e recebe de volta o codigo
		try {
			codigo = dao.create(codigo, nome, preco, quantidade);
		} catch (DataAccessException e) {
			throw new RemoteException("Falha ao criar: " + e);
		}
		
		// atualiza os atributos de dados deste objeto com os do banco
		load(codigo);
	}

	// Metodos de negocio
	
	public String getCodigo()  throws RemoteException {
		System.out.println("ProdutoImpl: getCodigo()");
		return codigo;
	}
	public void setCodigo(String codigo)  throws RemoteException {
		System.out.println("ProdutoImpl: setCodigo("+codigo+")");
		this.codigo = codigo;
		save();
	}
	
	public void setNome(String nome)  throws RemoteException {
		System.out.println("ProdutoImpl: setNome("+nome+")");
		this.nome = nome;
		save();
	}
	public String getNome()  throws RemoteException {
		System.out.println("ProdutoImpl: getNome()");
		return nome;
	}
	
	public void setPreco(BigDecimal preco)  throws RemoteException {
		System.out.println("ProdutoImpl: setPreco("+preco+")");
		this.preco = preco;
		save();
	}
	public BigDecimal getPreco()  throws RemoteException {
		System.out.println("ProdutoImpl: getPreco()");
		return preco;
	}
	
	public int getQuantidade()  throws RemoteException {
		System.out.println("ProdutoImpl: getQuantidade()");
		return quantidade;
	}
	public void setQuantidade(int quantidade)  throws RemoteException {
		System.out.println("ProdutoImpl: setQuantidade("+quantidade+")");
		this.quantidade = quantidade;
		save();
	}
	
	public void addItens(int quantidade)  throws RemoteException {
		System.out.println("ProdutoImpl: addItens("+quantidade+")");
		this.quantidade += quantidade;
		save();
	}
	
	// Metodos de sincronizacao com o banco
	
	/**
	 * Carrega o objeto do banco e atualiza as variaveis
	 * de instancia deste objeto.
	 */
	private void load(String codigo) throws RemoteException {
		System.out.println("ProdutoImpl: load("+codigo+")");
		ProdutoVO valueObject = null;
		try {
			valueObject = dao.load(codigo);
		} catch (DataAccessException e) {
			throw new RemoteException("Falha ao carregar dados: " + e);
		}
		this.codigo = valueObject.getCodigo();
		this.nome = valueObject.getNome();
		this.preco = valueObject.getPreco();
		this.quantidade = valueObject.getQuantidade();
	}
	
	/**
	 * Persiste o estado deste objeto no banco.
	 */
	public void save() throws RemoteException {
		System.out.println("ProdutoImpl: save()");
		try {
			dao.store(codigo, nome, preco, quantidade);
		} catch (DataAccessException e) {
			throw new RemoteException("Falha ao armazenar dados: " + e);
		}
	}

}
