package loja;

import javax.ejb.*;
import java.util.*;

public class ProdutoBean implements EntityBean {

	protected EntityContext ctx;
	
	private String codigo;
	private String nome;
	private double preco;
	private int quantidade;
	
	private ProdutoDAO dao;

	public ProdutoBean() {
		System.out.println("Construtor ProdutoBean() chamado!");
		try {
			dao = new ProdutoDAO();
		} catch (DataAccessException e) {
			throw new EJBException("ProdutoBean-<init>(): " + e);
		}
	}

	public String getNome() {
		return nome;
    }
	public void setNome(String name){
		this.nome = nome;
    }
    public String getCodigo(){
        return codigo;
    }
	public void setCodigo(String codigo){
		this.codigo = codigo;
    }
	public double getPreco(){
        return preco;
    }
	public void setPreco(double preco){
		this.preco = preco;
    }
	public int getQuantidade(){
        return quantidade;
    }
	public void setQuantidade(int quantidade){
		this.quantidade = quantidade;
    }
	public void addItens(int quantidade) {
		this.quantidade += quantidade;
	}

	public void ejbActivate() {
		System.out.println("ejbActivate() called.");
	}
	public void ejbPassivate() {
		System.out.println("ejbPassivate () called.");
	}
	public void setEntityContext(EntityContext ctx) {
		System.out.println("setEntityContext called");
		this.ctx = ctx;
	}
	public void unsetEntityContext() {
		System.out.println("unsetEntityContext called");
		this.ctx = null;
	}

	public void ejbRemove() {
		System.out.println("ejbRemove() called.");
		ProdutoPK pk = (ProdutoPK) ctx.getPrimaryKey();
		try {
			dao.remove(pk.codigo);
		} catch (DataAccessException e) {
			throw new EJBException("ProdutoBean-ejbRemove(): " + e);
		}
	}

	public void ejbLoad() {
		System.out.println("ejbLoad() called.");
		ProdutoPK pk = (ProdutoPK) ctx.getPrimaryKey();
		try {
			ProdutoVO vo = dao.load(pk.codigo);
			codigo = vo.getCodigo();
			nome = vo.getNome();
			preco = vo.getPreco();
			quantidade = vo.getQuantidade();
		} catch (DataAccessException e) {
			throw new EJBException("ProdutoBean-ejbLoad(): " + e);
		}
	}

	public void ejbStore() {
		System.out.println("ejbStore() called.");
		try {
			dao.store(codigo, nome, preco, quantidade);
		} catch (DataAccessException e) {
			throw new EJBException("ProdutoBean-ejbStore(): " + e);
		}
	}

	 public void ejbPostCreate(String codigo, String nome, double preco, int quantidade) {
		System.out.println("ProdutoBean-ejbPostCreate()");
	 }
  	 public ProdutoPK ejbCreate(String codigo, String nome, double preco, int quantidade) throws CreateException {
		System.out.println("ejbCreate() called");
		try {
			dao.create(codigo, nome, preco, quantidade);
			this.codigo = codigo;
			this.nome = nome;
			this.preco = preco;
			this.quantidade = quantidade;
		} catch (DataAccessException e) {
			throw new CreateException("ProdutoBean-ejbCreate(): " + e);
		}
		return new ProdutoPK(codigo);
	}

	/** Finders **/
	
	public ProdutoPK ejbFindByPrimaryKey(ProdutoPK key) throws FinderException {
		try {
			if (!dao.exists(codigo)) {
				throw new ObjectNotFoundException("ProdutoBean-ejbRemove(): Codigo nao existe!");
			}
		} catch (DataAccessException e) {
			throw new FinderException("ProdutoBean-ejbFindByPrimaryKey(): " + e);
		}
		return key;
	}
	
    public Collection ejbFindByNome(String name) throws FinderException {
		try {
			String[] codigos = dao.getByNome(nome);
			List list = new ArrayList();
			for (int i = 0; i < codigos.length; i++) {
				list.add(new ProdutoPK(codigos[i]));
			}
			return list;
		} catch (DataAccessException e) {
			throw new FinderException("ProdutoBean-ejbFindByNome(): " + e);
		}
	}
    public Collection ejbFindByPreco(double min, double max) throws FinderException {
		try {
			String[] codigos = dao.getByPreco(min, max);
			List list = new ArrayList();
			for (int i = 0; i < codigos.length; i++) {
				list.add(new ProdutoPK(codigos[i]));
			}
			return list;
		} catch (DataAccessException e) {
			throw new FinderException("ProdutoBean-ejbFindByPreco(): " + e);
		}
	}
    public Collection ejbFindAllProdutos() throws FinderException {
		try {
			String[] codigos = dao.getAll();
			List list = new ArrayList();
			for (int i = 0; i < codigos.length; i++) {
				list.add(new ProdutoPK(codigos[i]));
			}
			return list;
		} catch (DataAccessException e) {
			throw new FinderException("ProdutoBean-ejbFindAllProdutos(): " + e);
		}
	}

	public String ejbHomeGetProximoCodigo() {
		try {
			return dao.criarCodigo();
		} catch (DataAccessException e) {
			throw new EJBException("ProdutoBean-ejbFindAllProdutos(): " + e);
		}
	}

}

