package loja;

import loja.*;

import java.sql.*;
import java.util.*;

import javax.naming.*;
import javax.sql.*;

/**
<pre>
create table produtos (
	id integer constraint pk_produtos primary key, 
	nome varchar(64), 
	preco numeric(10,2),
	qte   integer
);
</pre>
*/
public class ProdutoDAO {
	
	/** SQL queries */
	public static final String SQL_CODIGO = "select max(id) from produtos";
	public static final String SQL_CREATE = "insert into produtos(id, nome, preco, qte) values (?, ?, ?, ?)";
	public static final String SQL_REMOVE = "delete from produtos where id = ?";
	public static final String SQL_LOAD   = "select id, nome, preco, qte from produtos where id = ?";
	public static final String SQL_STORE  = "update produtos set nome = ?, preco = ?, qte = ? where id = ?";
	public static final String SQL_PK     = "select id from produtos where id = ?";
	public static final String SQL_ALL    = "select id from produtos";
	public static final String SQL_NOME   = "select id from produtos where nome = ?";
	public static final String SQL_PRECO  = "select id from produtos where preco >= ? and preco <= ?";
	
	//private String user = "sa";
	//private String pass = "";
	//private String url = "jdbc:hsqldb:hsql://localhost:1701";
	//private String driverClass = "org.hsqldb.jdbcDriver";
	private DataSource dataSource;
	
	public ProdutoDAO() throws DataAccessException {
		try {
			Context ctx = new InitialContext();
			dataSource = (DataSource)ctx.lookup("java:comp/env/jdbc/produtoDB");
		} catch (NamingException e) {
			throw new DataAccessException("Nao foi possivel iniciar o banco: " + e);
		}
	}
	
	private Connection getConnection() throws DataAccessException {
		Connection con = null;
		try {
			con = dataSource.getConnection();
		} catch (java.sql.SQLException e) {
			throw new DataAccessException("Nao foi possivel iniciar a conexao!");
		}
		return con;
	}
	
	/**
	 * Cria uma nova PK.
	 */
	public String criarCodigo() throws DataAccessException {
		String sql = SQL_CODIGO;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		String codigo = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				int ultimo = rs.getInt(1);
				codigo = "" + (ultimo + 1);
			} else {
				codigo = "1";
			}
			if (codigo == null) {
				throw new SQLException("Incapaz de criar novo codigo!");
			}
		} catch (SQLException e) {
			throw new DataAccessException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
		return codigo;
	}
	
	public String create(String codigo, String nome, double preco, int quantidade) throws DataAccessException {
		String sql = SQL_CREATE;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setInt(1, Integer.parseInt(codigo));
			stmt.setString(2, nome);
			stmt.setDouble(3, preco);
			stmt.setInt(4, quantidade);
			stmt.executeUpdate();
		} catch (SQLException e) {
			throw new DataAccessException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
		return codigo;
	}
	
	public void remove(String codigo) throws DataAccessException {
		String sql = SQL_REMOVE;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setInt(1, Integer.parseInt(codigo));
			stmt.executeUpdate();
		} catch (SQLException e) {
			throw new DataAccessException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
	
	public ProdutoVO load(String codigo) throws DataAccessException  {
		String sql = SQL_LOAD;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		ProdutoVO produto = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setInt(1, Integer.parseInt(codigo));
			ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				produto = new ProdutoVO(""+rs.getInt(1),
										  rs.getString(2),
										  rs.getDouble(3),
										  rs.getInt(4));
			}
		} catch (SQLException e) {
			throw new DataAccessException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
		return produto;
	}
	
	public void store(String codigo, String nome, double preco, int quantidade) throws DataAccessException {
		String sql = SQL_STORE;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setString(1, nome);
			stmt.setDouble(2, preco);
			stmt.setInt(3, quantidade);
			stmt.setInt(4, Integer.parseInt(codigo));
			stmt.executeUpdate();
		} catch (SQLException e) {
			throw new DataAccessException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
	
	public boolean exists(String codigo) throws DataAccessException {
		String sql = SQL_PK;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		int codigoLido = -1;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setInt(1, Integer.parseInt(codigo));
			ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				codigoLido = rs.getInt(1);
			}
			if (codigoLido == -1) {
				throw new SQLException("Registro " + codigo + " inexistente!");
			}
			return true;
		} catch (SQLException e) {
			throw new DataAccessException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) { return false; }
		}
	}
	
	public String[] getAll() throws DataAccessException {
		String sql = SQL_ALL;
		System.out.println("Executando " + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		List codigoList = new ArrayList();
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			ResultSet rs = stmt.executeQuery();
			while (rs.next()) {
				codigoList.add(""+rs.getInt(1));
			}
			String[] buffer = new String[codigoList.size()];
			return (String[]) codigoList.toArray(buffer);
		} catch (SQLException e) {
			throw new DataAccessException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
	
	public String[] getByNome(String nome) throws DataAccessException {
		String sql = SQL_NOME;
		System.out.println("Executando " + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		List codigoList = new ArrayList();
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setString(1, nome);
			ResultSet rs = stmt.executeQuery();
			while (rs.next()) {
				codigoList.add(""+rs.getInt(1));
			}
			String[] buffer = new String[codigoList.size()];
			return (String[]) codigoList.toArray(buffer);
		} catch (SQLException e) {
			throw new DataAccessException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
	
	public String[] getByPreco(double min, double max) throws DataAccessException {
		String sql = SQL_PRECO;
		System.out.println("Executando " + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		List codigoList = new ArrayList();
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setDouble(1, min);
			stmt.setDouble(2, max);
			ResultSet rs = stmt.executeQuery();
			while (rs.next()) {
				codigoList.add(""+rs.getInt(1));
			}
			String[] buffer = new String[codigoList.size()];
			return (String[]) codigoList.toArray(buffer);
		} catch (SQLException e) {
			throw new DataAccessException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
}
