package loja.ejb;

import javax.ejb.*;
import java.util.*;
import java.math.BigDecimal;

import loja.integration.*;
import loja.*;

public class ItemBean implements EntityBean {
	
	protected EntityContext ctx;
	
	private String numero;
	private ProdutoLocal produto;
	private String compraId;
	private int quantidade;
	
	private ProdutoLocalHome produtoHome;
	
	private ItemDAO itemDAO;
	
	//////////////////// life-cycle ////////////////////////////
	
	/////////// 2. Home ///////////////
	
	public void  ejbPostCreate(String numero, ProdutoLocal produto, String compraId) {}
	public ItemPK ejbCreate(String numero, ProdutoLocal produto, String compraId) throws CreateException {
		return ejbCreate(numero, produto, compraId, 1);
	}
	
	public void  ejbPostCreate(String numero, ProdutoLocal produto, String compraId, int quantidade) {}
	public ItemPK ejbCreate(String numero, ProdutoLocal produto, String compraId, int quantidade) throws CreateException {
		System.out.println("ItemBean: ejbCreate("+numero+","+ produto +","+ compraId+","+ quantidade+")");
		this.numero = numero;
		this.produto = produto;
		this.compraId = compraId;
		this.quantidade = quantidade;
		try {
			itemDAO = JdbcItemDAO.getInstance();
			ItemVO item = new ItemVO(numero, produto.getCodigo(), compraId, quantidade);
			return new ItemPK(itemDAO.create(item));
		} catch (LojaException e) {
			throw new CreateException("Incapaz de obter instancia DAO: " + e);
		}
	}
	
	public ItemPK ejbFindByPrimaryKey(ItemPK pk) throws FinderException {
		System.out.println("ItemBean: ejbFindByPrimaryKey()");
		try {
			itemDAO = JdbcItemDAO.getInstance();
			if (itemDAO.exists(pk.numero)) {
				return pk;
			} else {
				throw new ObjectNotFoundException("Objeto " + pk.numero + " nao existe!");
			}
		} catch (LojaException e) {
			throw new EJBException("Falha na comunicacao com o banco: " + e);
		}
	}
	
	public Collection ejbFindAll() throws FinderException {
		System.out.println("ItemBean: ejbFindAll()");
		String[] numeros = null;
		try {
			itemDAO = JdbcItemDAO.getInstance();
			numeros = itemDAO.getAll();
		} catch (LojaException e) {
			throw new EJBException("Falha na comunicacao com o banco: " + e);
		}
		List list = new ArrayList(numeros.length);
		for (int i = 0; i < numeros.length; i++) {
			list.add(new ItemPK(numeros[i]));
		}
		return list;
	}
	
	
	public Collection ejbFindByCompra(String numero) throws FinderException {
		System.out.println("ItemBean: ejbFindByCompra("+numero+")");
		String[] numeros = null;
		try {
			itemDAO = JdbcItemDAO.getInstance();
			numeros = itemDAO.getByCompra(numero);
		} catch (LojaException e) {
			throw new EJBException("Falha na comunicacao com o banco: " + e);
		}
		List list = new ArrayList(numeros.length);
		for (int i = 0; i < numeros.length; i++) {
			list.add(new ItemPK(numeros[i]));
		}
		return list;
	}
	
	public Collection ejbFindByProduto(String codigo) throws FinderException {
		System.out.println("ItemBean: ejbFindByProduto("+codigo+")");
		String[] numeros = null;
		try {
			itemDAO = JdbcItemDAO.getInstance();
			numeros = itemDAO.getByProduto(codigo);
		} catch (LojaException e) {
			throw new EJBException("Falha na comunicacao com o banco: " + e);
		}
		List list = new ArrayList(numeros.length);
		for (int i = 0; i < numeros.length; i++) {
			list.add(new ItemPK(numeros[i]));
		}
		return list;
	}
	
	/////////// 2. Persistencia e container call-back ///////////////////
	public void ejbLoad() {
		System.out.println("ItemBean: ejbLoad()");
		ItemPK pk = (ItemPK)ctx.getPrimaryKey(); 
		ItemVO item = null;
		try {
			itemDAO = JdbcItemDAO.getInstance();
			item = itemDAO.load(pk.numero);
		} catch (LojaException e) {
			throw new EJBException("Falha na comunicacao com o banco: " + e);
		}
		// Nao use setXXX() para nao causar a execucao de store
		numero = item.getNumero();
		compraId  = item.getCompraId();
		quantidade = item.getQuantidade();
		
		String produtoId   = item.getProdutoId();
		try {
			produto = produtoHome.findByPrimaryKey(new ProdutoPK(produtoId));
		} catch (FinderException e) {
			throw new EJBException("Objeto de relacionamento nao encontrado: " + e);
		}
	}
	
	public void ejbStore() {
		System.out.println("ItemBean: ejbStore()");
		ItemVO item = new ItemVO(numero, produto.getCodigo(), compraId, quantidade);
		try {
			itemDAO = JdbcItemDAO.getInstance();
			itemDAO.store(item);
		} catch (LojaException e) {
			throw new EJBException("Falha na comunicacao com o banco: " + e);
		}
	}
	
	public void ejbRemove() {
		System.out.println("ItemBean: ejbRemove()");
		ItemPK pk = (ItemPK)ctx.getPrimaryKey();
		try {
			itemDAO = JdbcItemDAO.getInstance();
			itemDAO.remove(pk.numero);
		} catch (LojaException e) {
			throw new EJBException("Falha na comunicacao com o banco: " + e);
		}
	}
	
	public String ejbHomeGetProximoNumero() {
		try {
			itemDAO = JdbcItemDAO.getInstance();
			return itemDAO.criarNumero();
		} catch (LojaException e) {
			throw new EJBException("Incapaz de criar numero: " + e);
		}
	}
	
	
	public void setEntityContext(EntityContext ctx) {
		System.out.println("ItemBean: setEntityContext()");
		try {
			ServiceLocator locator = ServiceLocator.getInstance();
			produtoHome = (ProdutoLocalHome)locator.findLocalEJB("ejb/loja/produto");
		} catch (LojaException e) {
			throw new EJBException("Objeto de relacionamento nao encontrado: " + e);
		}
		this.ctx = ctx;
	}
	public void unsetEntityContext() {
		ctx = null;
	}
	
	public void ejbActivate() {}
	public void ejbPassivate() {}
	
	
	//////////// BUSINESS AND ACESSOR ///////////////
	public String getNumero() {
		System.out.println("ItemBean: getNumero()");
		return numero;
	}
	public void setNumero(String numero) {
		System.out.println("ItemBean: setNumero("+numero+")");
		this.numero = numero;
	}
	
	public void setProduto(ProdutoLocal produto) {
		System.out.println("ItemBean: setProduto("+produto+")");
		this.produto = produto;
	}
	public ProdutoLocal getProduto() {
		System.out.println("ItemBean: getProduto()");
		return produto;
	}
	
	public void setCompra(String compraId) {
		System.out.println("ItemBean: setCompraId("+compraId+")");
		this.compraId = compraId;
	}
	public String getCompra() {
		System.out.println("ItemBean: getCompraId()");
		return compraId;
	}
	
	public int getQuantidade() {
		System.out.println("ItemBean: getQuantidade()");
		return quantidade;
	}
	public void setQuantidade(int quantidade) {
		System.out.println("ItemBean: setQuantidade("+quantidade+")");
		this.quantidade = quantidade;
	}
	
	public double getSubTotal() {
		System.out.println("ItemBean: getSubTotal()");
		return produto.getPreco() * quantidade;
	}

}
