package loja;

import loja.integration.ServiceLocator;
import loja.ejb.*;
import java.util.Collection;

import java.math.BigDecimal;

public class LojaBusinessDelegate implements BusinessDelegate {
	
	LojaSessionFacade facade;
	
	public LojaBusinessDelegate() throws LojaException {
		ServiceLocator locator = ServiceLocator.getInstance();
		LojaSessionFacadeHome home = (LojaSessionFacadeHome)
			locator.findRemoteEJB("ejb/loja/main", LojaSessionFacadeHome.class);
		try {
			facade = home.create();
			if (facade == null) {
				throw new LojaException("Fachada eh null!");
			}
		} catch (javax.ejb.CreateException e) {
			throw new LojaException(e);
		} catch (java.rmi.RemoteException e) {
			throw new LojaException(e);
		}
	}

	public void criarCarrinho() throws LojaException {
		try {
			facade.criarCarrinho();
		} catch (java.rmi.RemoteException e) {
			throw new LojaException(e);
		} 
	}
	
	public ItemVO[] listarConteudoCarrinho() throws LojaException {
		try {
			CarrinhoSession carrinho = facade.getCarrinho();
			return carrinho.getConteudo();
		} catch (java.rmi.RemoteException e) {
			throw new LojaException(e);
		} 
	}
	public void removerProdutosCarrinho(String codigo) throws LojaException {
		try {
			CarrinhoSession carrinho = facade.getCarrinho();
			carrinho.removerProdutos(codigo);
		} catch (java.rmi.RemoteException e) {
			throw new LojaException(e);
		} 
	}
	public void esvaziarCarrinho() throws LojaException {
		try {
			CarrinhoSession carrinho = facade.getCarrinho();
			carrinho.esvaziar();
		} catch (java.rmi.RemoteException e) {
			throw new LojaException(e);
		} 
	}
	public void adicionarProdutosCarrinho(String codigo) throws LojaException {
		try {
			CarrinhoSession carrinho = facade.getCarrinho();
			carrinho.adicionarProdutos(codigo, 1);
		} catch (java.rmi.RemoteException e) {
			throw new LojaException(e);
		} 
	}
	public void adicionarProdutosCarrinho(String codigo, int quantidade) throws LojaException {
		try {
			CarrinhoSession carrinho = facade.getCarrinho();
			carrinho.adicionarProdutos(codigo, quantidade);
		} catch (java.rmi.RemoteException e) {
			throw new LojaException(e);
		} 
	}
	
	public ProdutoVO[] listarProdutosDisponiveis() throws LojaException {
		try {
			return facade.listarProdutosDisponiveis();
		} catch (java.rmi.RemoteException e) {
			throw new LojaException(e);
		} 
	}
	
	public CompraVO processarCompra(String userid, ItemVO[] carrinho) throws LojaException {
		try {
			return facade.processarCompra(userid, carrinho);
		} catch (java.rmi.RemoteException e) {
			throw new LojaException(e);
		} 
	}
	
	public String comprar() throws LojaException  {
		try {
			return facade.comprar();
		} catch (java.rmi.RemoteException e) {
			throw new LojaException(e);
		} 
	}

}
