package loja;

import java.util.*;

/**
 * Typesafe enum para representar estados que um pedido de compra pode representar.
 */
public final class Status implements java.io.Serializable {

	// Constantes publicas
	/** Um novo pedido de compra foi criado. Aguarda confirmao do cliente para dar andamento. */
	public final static Status INICIADO  	 = new Status("Iniciado", "Um novo pedido de compra foi criado. Aguarda confirmao do cliente para dar andamento.");
	
	/** Pedido de compra em andamento. Solicitao foi enviada  operadora do carto e aguarda-se confirmao */
	public final static Status EM_ANDAMENTO  = new Status("Em andamento", "Pedido de compra em andamento. Solicitao foi enviada  operadora do carto e aguarda-se confirmao.");
	
	/** Este pedido de compra foi cancelado durante o andamento e ser removido. */
	public final static Status ABORTADO  	 = new Status("Abortado", "Este pedido de compra foi cancelado durante o andamento e ser removido.");
	
	/** Este pedido de compra foi temporariamente suspenso (talvez devido a no aprovao do carto). Ele pode ser reiniciado com novos dados. */
	public final static Status SUSPENSO  	 = new Status("Suspenso", "Este pedido de compra foi temporariamente suspenso (talvez devido a no aprovao do carto). Ele pode ser reiniciado com novos dados. ");
	
	/** O pedido foi processado, dados do cliente foram aprovados, pagamento realizado e itens enviados. */
	public final static Status CONCLUIDO 	 = new Status("Concluido", "O pedido foi processado, dados do cliente foram aprovados, pagamento realizado e itens enviados.");
	
	/** O pedido foi cancelado pelo cliente. */
	public final static Status CANCELADO 	 = new Status("Cancelado", "O pedido foi cancelado pelo cliente.");
	
	/** Indica erro na aplicao. */
	public final static Status IRREGULAR 	 = new Status("Irregular", "Indica erro na aplicao.");
	
	// Inicializacao estatica de mapa para viabilizar a criacao de objetos Status
	private static HashMap statusMap = new HashMap();
	static {
		statusMap.put(Status.INICIADO.toString(), Status.INICIADO);
		statusMap.put(Status.EM_ANDAMENTO.toString(), Status.EM_ANDAMENTO);
		statusMap.put(Status.ABORTADO.toString(), Status.ABORTADO);
		statusMap.put(Status.SUSPENSO.toString(), Status.SUSPENSO);
		statusMap.put(Status.CONCLUIDO.toString(), Status.CONCLUIDO);
		statusMap.put(Status.CANCELADO.toString(), Status.CANCELADO);
		statusMap.put(Status.IRREGULAR.toString(), Status.IRREGULAR);
	}

	/**
	 * Retorna objeto Status correspondente a um estado expresso em forma de string
	 */
	public static Status getStatus(String status) {
		if (statusMap.containsKey(status)) {
			return (Status)statusMap.get(status);
		}
		return Status.IRREGULAR;
	}
	
	/**
	 * Retorna todos os objetos Status disponiveis
	 */
	 public static Status[] allStatus() {
		 Collection all = statusMap.values();
		 Status[] buffer = new Status[all.size()];
		 return (Status[]) all.toArray(buffer);
	 }			 
	
	/**
	 * Retorna detalhes sobre o estado.
	 */
	public String getInfo() {
		return info;
	}
	
	// Typesafe enum private constructor
	private String status, info;
	private Status(String status, String info) {
		this.status = status;
		this.info = info;
	}
	
	public int hashCode() {
		return status.hashCode();
	}
	public boolean equals(Object o) {
		return ((Status)o).status.equals(status);
	}
	public String toString() {
		return status;
	}
	
	
}
