package loja.client;

import loja.*;
import loja.security.*;
import java.util.*;

import java.math.BigDecimal;

public class LojaClient {
	
	public static void main(String[] args) throws LojaException {
		LojaBusinessDelegate loja = new LojaBusinessDelegate();
		
		// 1. Teste produtos disponiveis
		ProdutoVO[] produtos = loja.listarProdutosDisponiveis();
		List codigos = new ArrayList();
		for (int i = 0; i < produtos.length; i++) {
			codigos.add(produtos[i].getCodigo());
			System.out.println(produtos[i]);
		}
		
		// 2. Teste carrinho de compras
		loja.criarCarrinho();
		
		ItemVO[] carrinho = loja.listarConteudoCarrinho();
		System.out.println("Carrinho tem " + carrinho.length + " elementos.");
		for(int i = 0; i < carrinho.length; i++) {
			System.out.println(carrinho[i]);
		}
		
		System.out.println("Adicionando itens...");
		if (codigos.size() >= 3) {
			System.out.println("Um produto 1");
			loja.adicionarProdutosCarrinho( (String)codigos.get(0) );
			System.out.println("Outro produto 1");
			loja.adicionarProdutosCarrinho( (String)codigos.get(0) );
			System.out.println("Mais 7 produtos 1 - total: 9");
			loja.adicionarProdutosCarrinho( (String)codigos.get(0), 7); // total 9
			System.out.println("4 produtos 2");
			loja.adicionarProdutosCarrinho( (String)codigos.get(1), 4);
			System.out.println("11 produtos 3");
			loja.adicionarProdutosCarrinho( (String)codigos.get(2), 11);
		} else {
			// criar alguns produtos se necessario
			AdminLojaBusinessDelegate admin = new AdminLojaBusinessDelegate();
		
			if (admin.listarProdutos().length < 3) {
				admin.criarProduto("Tiranossauro", new BigDecimal(13500.00));
				admin.criarProduto("Computador", new BigDecimal(800.00));
				admin.criarProduto("Pizza", new BigDecimal(22.00));
			}
			
			// disponibilizar produtos para compra
			System.out.println("Encomendando 6 produtos de codigo [1].");
			admin.solicitarCompra("1", 6);
			System.out.println("Encomendando 11 produtos de codigo [2].");
			admin.solicitarCompra("2", 11);
			System.out.println("Encomendando 24 produtos de codigo [3].");
			admin.solicitarCompra("3", 24);
			
			produtos = loja.listarProdutosDisponiveis();
			if (produtos.length < 3) {
				System.out.println("Esperando 5 segundos para o MDB processar os pedidos...");
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {}
			}
			
			System.out.println("Produtos Disponiveis para compra: ");
			produtos = loja.listarProdutosDisponiveis();
			if (produtos.length < 3) {
				System.out.println("MDB ainda nao criou os objetos - tente mais tarde.");
				return;
			}
			codigos = new ArrayList();
			for (int i = 0; i < produtos.length; i++) {
				codigos.add(produtos[i].getCodigo());
				System.out.println(produtos[i]);
			}
			
			System.out.println("Acrescentando produtos no carrinho");
			loja.adicionarProdutosCarrinho( (String)codigos.get(0), 9);
			loja.adicionarProdutosCarrinho( (String)codigos.get(1), 4);
			loja.adicionarProdutosCarrinho( (String)codigos.get(2), 11);
		}
		
		carrinho = loja.listarConteudoCarrinho();
		System.out.println("Carrinho tem " + carrinho.length + " elementos.");
		for(int i = 0; i < carrinho.length; i++) {
			System.out.println(carrinho[i]);
		}
		
		System.out.println("Removendo todos os produtos 1...");
		loja.removerProdutosCarrinho( (String)codigos.get(0) );

		carrinho = loja.listarConteudoCarrinho();
		System.out.println("Carrinho tem " + carrinho.length + " elementos.");
		for(int i = 0; i < carrinho.length; i++) {
			System.out.println(carrinho[i]);
		}
		
		System.out.println("Esvaziando o carrinho completamente...");
		loja.esvaziarCarrinho();

		carrinho = loja.listarConteudoCarrinho();
		System.out.println("Carrinho tem " + carrinho.length + " elementos.");
		for(int i = 0; i < carrinho.length; i++) {
			System.out.println(carrinho[i]);
		}
		
		// 3. Teste fechamento de compra - principal eh cliente
		
		// Acrescentando produtos no carrinho primeiro
		System.out.println("Teste do processamento do pedido");
		System.out.println("Adicionando itens outra vez no carrinho.");
		loja.adicionarProdutosCarrinho( (String)codigos.get(0), 5);
		loja.adicionarProdutosCarrinho( (String)codigos.get(1), 4);
		loja.adicionarProdutosCarrinho( (String)codigos.get(2), 20);
		
		carrinho = loja.listarConteudoCarrinho();
		System.out.println("Carrinho tem " + carrinho.length + " elementos.");
		for(int i = 0; i < carrinho.length; i++) {
			System.out.println(carrinho[i]);
		}
		
		// Prosseguindo com o teste
		boolean aceito = false;
		LoginBusinessDelegate login = new LoginBusinessDelegate();
		try {
			Authenticator.login("helder", "redleh");
			System.out.println("Cliente ja existe e foi aceito!");
			aceito = true;
		} catch (javax.security.auth.login.LoginException e) {
			System.out.println("Senha de helder estah incorreta ou usuario helder nao existe: " + e);
		}
		if (!aceito) {
			System.out.println("Cliente nao existe. Tentarei cadastra-lo!");
			ClienteVO cliente = new ClienteVO("helder", "redleh", "admin", "Helder da Rocha", "helder@localhost", "Rua Java, 530 - Sao Paulo", "1234567890123456");
			try {
				login.cadastrar(cliente);
				System.out.println("Cliente cadastrado!");
			} catch (UsuarioJaExisteException e) {
				System.out.println(""+e);
			}
			try {
				Authenticator.login("helder", "redleh");
				aceito = true;
			} catch (javax.security.auth.login.LoginException e) {
				System.out.println("" + e);
			}
		}

		if (aceito) {
			System.out.println("Processando compra...");
			CompraVO nota = loja.processarCompra("helder", carrinho);
			System.out.println("Nota do pedido para aprovacao: \n" + nota);
			System.out.println("Nota aprovada. Realizando compra...");
			String recibo = null;
			try {
				recibo = loja.comprar();
				carrinho = null;
			} catch (CreditoNaoAprovadoException e) {
				recibo = "CREDITO NAO APROVADO - Transao cancelada!";
			}
			System.out.println("Recibo: \n" + recibo); // recido eh a nota em XML
		}
		
		AdminLojaBusinessDelegate admin = new AdminLojaBusinessDelegate();
		ProdutoVO[] estoque = admin.listarProdutos();
		System.out.println(">>>>>>>> Produtos no estoque:");
		for (int i = 0; i < estoque.length; i++) {
			System.out.println(estoque[i]);
		}

		// Teste com cliente sem credito
		
		// Acrescentando produtos no carrinho primeiro
		System.out.println("Teste do processamento do pedido com usuario sem credito");
		System.out.println("Adicionando itens outra vez no carrinho.");
		loja.adicionarProdutosCarrinho( (String)codigos.get(0), 9);
		loja.adicionarProdutosCarrinho( (String)codigos.get(1), 9);
		loja.adicionarProdutosCarrinho( (String)codigos.get(2), 9);
		
		carrinho = loja.listarConteudoCarrinho();
		System.out.println("Carrinho tem " + carrinho.length + " elementos.");
		for(int i = 0; i < carrinho.length; i++) {
			System.out.println(carrinho[i]);
		}
		
		try {
			Authenticator.login("lalau", "hehehe");
			System.out.println("Cliente ja existe e foi aceito!");
			aceito = true;
		} catch (javax.security.auth.login.LoginException e) {
			System.out.println("Senha de helder estah incorreta ou usuario helder nao existe: " + e);
		}
		if (!aceito) {
			System.out.println("Cliente nao existe. Tentarei cadastra-lo!");
			ClienteVO cliente = new ClienteVO("lalau", "hehehe", "admin", "Lalau", "helder@localhost", "Av. Confianca, 777 - Honestopolis", "8373927394739493");
			try {
				login.cadastrar(cliente);
				System.out.println("Cliente cadastrado!");
			} catch (UsuarioJaExisteException e) {
				System.out.println(""+e);
			}
			try {
				Authenticator.login("helder", "redleh");
				aceito = true;
			} catch (javax.security.auth.login.LoginException e) {
				System.out.println("" + e);
			}
		}

		if (aceito) {
			System.out.println("Processando compra...");
			CompraVO nota = loja.processarCompra("lalau", carrinho);
			System.out.println("Nota do pedido para aprovacao: \n" + nota);
			System.out.println("Nota aprovada. Realizando compra...");
			String recibo = null;
			try {
				recibo = loja.comprar();
			} catch (CreditoNaoAprovadoException e) {
				recibo = "CREDITO NAO APROVADO - Transao cancelada!";
			}
			System.out.println("Recibo: \n" + recibo); // recido eh a nota em XML
		}

		/*
		estoque = admin.listarProdutos();
		System.out.println(">>>>>>>> Produtos no estoque:");
		for (int i = 0; i < estoque.length; i++) {
			System.out.println(estoque[i]);
		}
		*/
		
		// Teste de cascade-delete
		/*
		try {
			System.out.println("Removendo Cliente lalau, compras e itens associados ...");
			admin.removerCliente("lalau"); // remove das tabelas usuario e cliente, compras e itens
		} catch (LojaException e) {
			System.out.println(e);
		}
		*/
	}
}
