package loja.ejb;

import loja.integration.*;
import loja.*;

import javax.ejb.*;
import javax.jms.*;
import java.math.BigDecimal;
import java.util.*;

public class AdminLojaSessionFacadeBean implements SessionBean {
	
	/**
	 * Fila do JBoss usada quando esta aplicao for chamada por
	 * clientes que nao usam o ENC (java:comp/env)
	 */
	private static final String FALLBACK_QUEUE_NAME = "queue/A";
	
	private ProdutoLocalHome produtoHome;
	private ClienteLocalHome clienteHome;
	private CompraLocalHome  compraHome; 
	private ItemLocalHome    itemHome; 
	private SessionContext ctx;
	
	public void ejbCreate() throws CreateException {
		System.out.println("AdminLojaSessionFacadeBean: ejbCreate()");
		try {
			ServiceLocator locator = ServiceLocator.getInstance();
			produtoHome = (ProdutoLocalHome) locator.findLocalEJB("ejb/loja/produto");
			compraHome  = (CompraLocalHome)  locator.findLocalEJB("ejb/loja/compra");
			itemHome    = (ItemLocalHome)    locator.findLocalEJB("ejb/loja/item");
			clienteHome = (ClienteLocalHome) locator.findLocalEJB("ejb/loja/cliente");
		} catch (LojaException e) {
			throw new CreateException("Service Locator failure: " + e);
		}
	}
	
	public String criarProduto(String nome, BigDecimal preco) throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: criarProduto("+ nome+", "+preco+")");
		ProdutoDAO dao = JdbcProdutoDAO.getInstance();
		String codigo = dao.criarCodigo();
		try {
			System.out.println("AdminLojaSessionFacadeBean: Criando produto codigo " + codigo);
			produtoHome.create(codigo, nome, preco, 0);
		} catch (CreateException e) {
			throw new LojaException("Incapaz de criar produto " + nome);
		}
		return codigo;
	}
	
	public void removerProduto(String codigo) throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: removerProduto("+ codigo+")");
		try {
			ProdutoLocal p = produtoHome.findByPrimaryKey(new ProdutoPK(codigo));
			if (p.getQuantidade() == 0) {
				p.remove();
			} else {
				throw new LojaException("Esvazie o estoque de produtos " + codigo + " antes de remover do banco.");
			}
		} catch (FinderException e) {
			throw new LojaException(e);
		} catch (RemoveException e) {
			throw new LojaException(e);
		}
	}
	public ProdutoVO[] listarProdutos() throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: listarProdutos()");
		try {
			Collection beans = produtoHome.findAll();
			System.out.println("AdminLojaSessionFacadeBean: listarProdutos():Achados " + beans.size() + " beans.");
			Iterator it = beans.iterator();
			List produtos = new ArrayList();
			while(it.hasNext()) {
				ProdutoLocal produtoBean = (ProdutoLocal) it.next();
				ProdutoVO produtoVO = new ProdutoVO();
				produtoVO.setCodigo(produtoBean.getCodigo());
				produtoVO.setNome(produtoBean.getNome());
				produtoVO.setPreco(produtoBean.getPreco());
				produtoVO.setQuantidade(produtoBean.getQuantidade());
				produtos.add(produtoVO);
			}
			ProdutoVO[] buffer = new ProdutoVO[produtos.size()];
			return (ProdutoVO[]) produtos.toArray(buffer);
		} catch (FinderException e) {
			throw new LojaException(e);
		}
	}
	
	public CompraVO[] listarCompras() throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: listarCompras()");
		try {
			Collection beans = compraHome.findAll();
			System.out.println("AdminLojaSessionFacadeBean: listarCompras(): Achados " + beans.size() + " beans.");
			return gerarArrayCompraVO(beans);
		} catch (FinderException e) {
			throw new LojaException(e);
		}
	}
	
	public CompraVO[] listarCompras(String userid) throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: listarCompras()");
		try {
			Collection beans = compraHome.findByCliente(userid);
			System.out.println("AdminLojaSessionFacadeBean: listarCompras("+userid+"): Achados " + beans.size() + " beans.");
			return gerarArrayCompraVO(beans);
		} catch (FinderException e) {
			throw new LojaException(e);
		}
	}
	
	private CompraVO[] gerarArrayCompraVO(Collection beans) throws LojaException {
		Iterator it = beans.iterator();
		List compras = new ArrayList();
		while(it.hasNext()) {
			CompraLocal compraBean = (CompraLocal) it.next();
			CompraVO compraVO = new CompraVO();
			String compraId = compraBean.getNumero();
			compraVO.setNumero(compraId);
			compraVO.setStatus(compraBean.getStatus());
			compraVO.setClienteId(compraBean.getClienteId());
			
			ItemLocal[] itens = compraBean.getItens();
			ItemVO[] itensVO = new ItemVO[itens.length];
			for (int i = 0; i < itens.length; i++) {
				ProdutoLocal p = itens[i].getProduto();
				ProdutoVO produtoVO = new ProdutoVO(p.getCodigo(), p.getNome(), p.getPreco(), p.getQuantidade());
				itensVO[i] = new ItemVO(itens[i].getNumero(), produtoVO, compraId, itens[i].getQuantidade());
			}
			compraVO.setItens(itensVO);
			compras.add(compraVO);
		}
		CompraVO[] buffer = new CompraVO[compras.size()];
		return (CompraVO[]) compras.toArray(buffer);
	}
	
	public ClienteVO[] listarClientes() throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: listarClientes()");
		try {
			Collection beans = clienteHome.findAll();
			System.out.println("AdminLojaSessionFacadeBean: listarClientes():Achados " + beans.size() + " beans.");
			Iterator it = beans.iterator();
			List clientes = new ArrayList();
			while(it.hasNext()) {
				ClienteLocal clienteBean = (ClienteLocal) it.next();
				String userid = clienteBean.getUserid();
				String senha = clienteBean.getSenha();
				String grupo = clienteBean.getGrupo();
				String nome = clienteBean.getNome();
				String email = clienteBean.getEmail();
				String endereco = clienteBean.getEndereco();
				String cartao = clienteBean.getCartao();
				ClienteVO clienteVO = new ClienteVO(userid, senha, grupo, nome, email, endereco, cartao);
				clientes.add(clienteVO);
			}
			ClienteVO[] buffer = new ClienteVO[clientes.size()];
			return (ClienteVO[]) clientes.toArray(buffer);
		} catch (FinderException e) {
			throw new LojaException(e);
		}
	}
	
	public ProdutoVO mostrarProduto(String codigo) throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: mostrarProduto("+ codigo+")");
		try {
			ProdutoLocal produtoBean = produtoHome.findByPrimaryKey(new ProdutoPK(codigo));
			String nome = produtoBean.getNome();
			BigDecimal preco = produtoBean.getPreco();
			int quantidade = produtoBean.getQuantidade();
			return new ProdutoVO(codigo, nome, preco, quantidade);
		} catch (FinderException e) {
			throw new LojaException(e);
		} 
	}
	public CompraVO mostrarCompra(String numero) throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: mostrarCompra("+ numero+")");
		try {
			CompraLocal compraBean = compraHome.findByPrimaryKey(new CompraPK(numero));
			CompraVO compraVO = new CompraVO();
			String compraId = compraBean.getNumero();
			compraVO.setNumero(compraId);
			compraVO.setStatus(compraBean.getStatus());
			compraVO.setClienteId(compraBean.getClienteId());
			ItemLocal[] itens = compraBean.getItens();
			ItemVO[] itensVO = new ItemVO[itens.length];
			for (int i = 0; i < itens.length; i++) {
				ProdutoLocal p = itens[i].getProduto();
				ProdutoVO produtoVO = new ProdutoVO(p.getCodigo(), p.getNome(), p.getPreco(), p.getQuantidade());
				itensVO[i] = new ItemVO(itens[i].getNumero(), produtoVO, compraId, itens[i].getQuantidade());
			}
			compraVO.setItens(itensVO);
			return compraVO;
		} catch (FinderException e) {
			throw new LojaException(e);
		} 
	}
	
	public void alterarStatus(String compraId, Status status) throws LojaException {
		try {
			CompraLocal compraBean = compraHome.findByPrimaryKey(new CompraPK(compraId));
			compraBean.setStatus(status);
		} catch (FinderException e) {
			throw new LojaException(e);
		} 
	}
	
	public ClienteVO mostrarCliente(String userid) throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: mostrarCliente("+ userid+")");
		try {
			ClienteLocal clienteBean = clienteHome.findByPrimaryKey(new ClientePK(userid));
			String senha = clienteBean.getSenha();
			String grupo = clienteBean.getGrupo();
			String nome = clienteBean.getNome();
			String email = clienteBean.getEmail();
			String endereco = clienteBean.getEndereco();
			String cartao = clienteBean.getCartao();
			return new ClienteVO(userid, senha, grupo, nome, email, endereco, cartao);
		} catch (FinderException e) {
			throw new LojaException(e);
		} 
	}
	
	public void solicitarCompra(String codigo, int quantidade) throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: solicitarCompra("+ codigo+", "+quantidade+")");
		try {
			ServiceLocator locator = ServiceLocator.getInstance();
			Queue queue = locator.findQueue("jms/loja/estoque", FALLBACK_QUEUE_NAME);
			QueueConnection con = locator.findDefaultJMSQueueConnection();
			
			QueueSession session = con.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
			String text = codigo +"|"+ quantidade;
			TextMessage message = session.createTextMessage(text);
			QueueSender sender = session.createSender(queue);
			sender.send(message);
		} catch (JMSException e) {
			throw new LojaException(e);
		}
	}
	
	public void esvaziarEstoque(String codigo) throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: esvaziarEstoque("+ codigo+")");
		try {
			ProdutoLocal p = produtoHome.findByPrimaryKey(new ProdutoPK(codigo));
			if (p.getQuantidade() != 0) {
				p.setQuantidade(0);
			}
		} catch (FinderException e) {
			throw new LojaException(e);
		}
	}
	
	public void removerCliente(String userid) throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: removerCliente("+ userid+")");
		try {
			ClienteLocal c = clienteHome.findByPrimaryKey(new ClientePK(userid));
			c.remove();
		} catch (FinderException e) {
			throw new LojaException(e);
		} catch (RemoveException e) {
			throw new LojaException(e);
		}
	}
	
	public void removerCompra(String codigo) throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: removerCliente("+ codigo+")");
		try {
			CompraLocal c = compraHome.findByPrimaryKey(new CompraPK(codigo));
			c.remove();
		} catch (FinderException e) {
			throw new LojaException(e);
		} catch (RemoveException e) {
			throw new LojaException(e);
		}
	}
	
	public void ejbActivate() {}
	public void ejbPassivate() {}
	public void ejbRemove() {}
	public void setSessionContext(SessionContext ctx) {
		this.ctx = ctx;
	}
}
