package loja.ejb;

import loja.integration.*;
import loja.*;

import javax.ejb.*;
import javax.jms.*;
import java.math.BigDecimal;
import java.util.*;

public class CarrinhoSessionBean implements SessionBean {
	
	private ProdutoLocalHome produtoHome;
	private SessionContext ctx;
	
	Map beans;
	Map quantidades;
	
	public void ejbCreate() throws CreateException {
		System.out.println("CarrinhoSessionBean: ejbCreate()");
		beans = new HashMap();
		quantidades = new HashMap();
		try {
			ServiceLocator locator = ServiceLocator.getInstance();
			produtoHome = (ProdutoLocalHome) locator.findLocalEJB("ejb/loja/produto");
		} catch (LojaException e) {
			throw new CreateException("Service Locator failure: " + e);
		}
	}

	/**
	 * Inclui produto no carrinho na quantidade passada como parametro. Se produto ja
	 * existe, a quantidade nova eh substituida. Se quantidade for zero, produto
	 * sera removido. Fallback para erros no formato da quantidade (nmeros negativos)
	 *  acrescentar uma unidade.
	 */
	public void adicionarProdutos(String codigo, int quantidade) throws LojaException {
		System.out.println("CarrinhoSessionBean: adicionarProdutos("+ codigo +"," + quantidade + ")");
		
		if (quantidade < 0) {
			quantidade = 1;
		}
		if (quantidade == 0) {
			this.removerProdutos(codigo);
			return;
		}
		
		ProdutoLocal produto = null;
		try {
			produto = produtoHome.findByPrimaryKey(new ProdutoPK(codigo));
		} catch (FinderException e) {
			throw new LojaException("CarrinhoSessionBean - adicionarProduto(): Produto " + codigo + " nao existe!");
		}
		beans.put(codigo, produto);
		quantidades.put(codigo, new Integer(quantidade));
	}
	
	/**
	 * Remove todos os produtos de um determinado tipo
	 */
	public void removerProdutos(String codigo) throws LojaException {
		System.out.println("CarrinhoSessionBean: removerProduto("+ codigo+")");
		Object removed = beans.remove(codigo);
		quantidades.remove(codigo);
		if (removed == null) {
			System.out.println("CarrinhoSessionBean - removerProdutos(): Produto " + codigo + " nao existe!");
			// do nothing - ignore if product to be removed does not exist
		}
		assert beans.size() == quantidades.size() : "CarrinhoSessionBean - adicionarProduto(): Map mismatch!";
	}
	
	public void esvaziar() throws LojaException {
		System.out.println("CarrinhoSessionBean: esvaziarCarrinho()");
		beans.clear();
		quantidades.clear();
	}
	
	public ItemVO[] getConteudo() throws LojaException  {
		System.out.println("CarrinhoSessionBean: listarConteudo()");
		Iterator it = beans.values().iterator();
		List itensDoCarrinho = new ArrayList();
		while(it.hasNext()) {
			ProdutoLocal produtoBean = (ProdutoLocal) it.next();
			ProdutoVO produtoVO = new ProdutoVO();
			produtoVO.setCodigo(produtoBean.getCodigo());
			produtoVO.setNome(produtoBean.getNome());
			produtoVO.setPreco(produtoBean.getPreco());
			produtoVO.setQuantidade(produtoBean.getQuantidade());
			
			Integer qte = (Integer) quantidades.get(produtoBean.getCodigo());
			ItemVO item = new ItemVO(produtoVO, qte.intValue());
			
			itensDoCarrinho.add(item);
		}
		ItemVO[] buffer = new ItemVO[itensDoCarrinho.size()];
		return (ItemVO[]) itensDoCarrinho.toArray(buffer);
	}
	
	public void ejbActivate() {}
	public void ejbPassivate() {}
	public void ejbRemove() {}
	public void setSessionContext(SessionContext ctx) {
		this.ctx = ctx;
	}
}
