package loja.ejb;

import javax.ejb.*;
import java.util.*;

import loja.integration.*;
import loja.*;

public class ClienteBean extends UsuarioBean {
	
	private String nome;
	private String email;
	private String endereco;
	private String cartao;
	
	private UsuarioLocalHome usuarioHome;
	
	private ClienteDAO clienteDAO;
	
	//////////////////// life-cycle ////////////////////////////
	
	//////////// 1. Contexto ///////////
	public void setEntityContext(EntityContext ctx) {
		super.setEntityContext(ctx);
		System.out.println("ClienteBean: setEntityContext()");
		try {
			ServiceLocator locator = ServiceLocator.getInstance();
			usuarioHome = (UsuarioLocalHome)locator.findLocalEJB("ejb/loja/usuario");
		} catch (LojaException e) {
			throw new EJBException("ClienteBean: Nao foi possivel encontrar objeto de relacionamento - setEntityContext(): " + e);
		}
	}

	/////////// 1. HOME ///////////////
	public void  ejbPostCreate(String userid, String senha, String grupo, String nome, String email, String endereco, String cartao) {
		super.ejbPostCreate(userid, senha, grupo);
	}
	public ClientePK ejbCreate(String userid, String senha, String grupo, String nome, String email, String endereco, String cartao) throws CreateException, UsuarioJaExisteException {
		System.out.println("ClienteBean: ejbCreate("+userid+","+ senha+","+ grupo+","+ nome+","+ endereco+","+ cartao+")");
		UsuarioPK usuarioPK = super.ejbCreate(userid, senha, grupo);
		this.nome = nome;
		this.email = email;
		this.endereco = endereco;
		this.cartao = cartao;
		try {
			clienteDAO = JdbcClienteDAO.getInstance();
			ClienteVO clienteVO = new ClienteVO(userid, senha, grupo, nome, email, endereco, cartao);
			return new ClientePK(clienteDAO.create(clienteVO));
		} catch (UsuarioJaExisteException e) {
			ctx.setRollbackOnly(); // doom this transaction
			throw e;
		} catch (LojaException e) {
			ctx.setRollbackOnly(); // doom this transaction
			throw new CreateException("ClienteBean: Incapaz de obter instancia DAO - ejbCreate(): " + e);
		}
	}

	public ClientePK ejbFindByPrimaryKey(ClientePK pk) throws FinderException {
		System.out.println("ClienteBean: ejbFindByPrimaryKey()");
		try {
			clienteDAO = JdbcClienteDAO.getInstance();
			if (clienteDAO.exists(pk.userid)) {
				return pk;
			} else {
				throw new ObjectNotFoundException("UsuarioBean - ejbFindByPrimaryKey(): Objeto " + pk.userid + " nao existe!");
			}
		} catch (LojaException e) {
			throw new EJBException("ClienteBean: Falha na comunicacao com o banco - ejbFindByPrimaryKey(): " + e);
		}
	}
	
	public Collection ejbFindAll() throws FinderException {
		System.out.println("ClienteBean: ejbFindAll()");
		String[] userids = null;
		try {
			clienteDAO = JdbcClienteDAO.getInstance();
			userids = clienteDAO.getAll();
		} catch (LojaException e) {
			throw new EJBException("ClienteBean: Falha na comunicacao com o banco - ejbFindAll(): " + e);
		}
		List list = new ArrayList(userids.length);
		for (int i = 0; i < userids.length; i++) {
			list.add(new ClientePK(userids[i]));
		}
		return list;
	}
	
	public Collection ejbFindByNome(String nome) throws FinderException {
		System.out.println("ClienteBean: ejbFindByNome("+nome+")");
		String[] userids = null;
		try {
			clienteDAO = JdbcClienteDAO.getInstance();
			userids = clienteDAO.getByNome(nome);
		} catch (LojaException e) {
			throw new EJBException("ClienteBean: Falha na comunicacao com o banco - ejbFindByNome(): " + e);
		}
		List list = new ArrayList(userids.length);
		for (int i = 0; i < userids.length; i++) {
			list.add(new ClientePK(userids[i]));
		}
		return list;
	}
	
	/////////// 2. CONTAINER CALL-BACK ///////////////////
	public void ejbLoad() {
		System.out.println("ClienteBean: ejbLoad()");
		super.ejbLoad();
		ClientePK pk = (ClientePK)ctx.getPrimaryKey(); 
		ClienteVO clienteVO = null;
		try {
			// Carrega dados da tabela de Clientes apenas
			clienteDAO = JdbcClienteDAO.getInstance();
			clienteVO = clienteDAO.load(pk.userid);
		} catch (LojaException e) {
			throw new EJBException("ClienteBean: Falha na comunicacao com o banco - ejbLoad(): " + e);
		}
		nome     = clienteVO.getNome();
		email    = clienteVO.getEmail();
		endereco = clienteVO.getEndereco();
		cartao   = clienteVO.getCartao();
	}
	
	public void ejbStore() {
		System.out.println("ClienteBean: ejbStore()");
		super.ejbStore();
		ClienteVO clienteVO = new ClienteVO(getUserid(), getSenha(), getGrupo(), nome, email, endereco, cartao);
		try {
			clienteDAO = JdbcClienteDAO.getInstance();
			clienteDAO.store(clienteVO);
		} catch (LojaException e) {
			throw new EJBException("ClienteBean: Falha na comunicacao com o banco - ejbStore(): " + e);
		}
	}
	
	/**
	 * Remove apenas os dados de cliente - usuario continua e pode logar!
	 */
	public void ejbRemove() {
		System.out.println("ClienteBean: ejbRemove()");
		ClientePK pk = (ClientePK)ctx.getPrimaryKey();
		
		// primeiro removemos as compras deste cliente
		CompraLocalHome compraHome = null;
		try {
			ServiceLocator locator = ServiceLocator.getInstance();
			compraHome = (CompraLocalHome)locator.findLocalEJB("ejb/loja/compra");
		} catch (LojaException e) {
			throw new EJBException("ClienteBean: Nao foi possivel encontrar objeto de relacionamento - setEntityContext(): " + e);
		}
		
		try {
			Collection compras = compraHome.findByCliente(pk.userid);
			Iterator it = compras.iterator();
			while(it.hasNext()) {
				CompraLocal compra = (CompraLocal)it.next();
				compra.remove();
			}
		} catch (RemoveException e) {
			throw new EJBException("ClienteBean: Cascade delete falhou: " + e);
		} catch (FinderException e) {
			throw new EJBException("ClienteBean: Cascade delete falhou: " + e);
		}

		// depois o cliente
		try {
			clienteDAO = JdbcClienteDAO.getInstance();
			clienteDAO.remove(pk.userid);
		} catch (LojaException e) {
			throw new EJBException("ClienteBean: Falha na comunicacao com o banco - ejbRemove(): " + e);
		}
		
		// por ultimo o usuario
		super.ejbRemove();
	}

	//////////// BUSINESS AND ACESSOR ///////////////
	
	// Mtodos herdados de Usuario
	public String getUserid() {
		System.out.println("ClienteBean: getUserid()");
		return super.getUserid();
	}
	public void setUserid(String userid) {
		System.out.println("ClienteBean: setUserid("+userid+")");
		super.setUserid(userid);
	}
	//////////////////////////////
	
	public String getNome() {
		System.out.println("ClienteBean: getNome()");
		return nome;
	}
	public void setNome(String nome) {
		System.out.println("ClienteBean: setNome("+nome+")");
		this.nome = nome;
	}
	
	public String getEmail() {
		System.out.println("ClienteBean: getEmail()");
		return email;
	}
	public void setEmail(String email) {
		System.out.println("ClienteBean: setEmail("+email+")");
		this.email = email;
	}
	
	public String getEndereco() {
		System.out.println("ClienteBean: getEndereco()");
		return endereco;
	}
	public void setEndereco(String endereco) {
		System.out.println("ClienteBean: setEndereco("+endereco+")");
		this.endereco = endereco;
	}
	
	public String getCartao() {
		System.out.println("ClienteBean: getCartao()");
		return cartao;
	}
	public void setCartao(String cartao) {
		System.out.println("ClienteBean: setCartao("+cartao+")");
		this.cartao = cartao;
	}

}
