package loja.ejb;

import loja.integration.*;
import loja.*;

import javax.ejb.*;
import javax.jms.*;
import java.math.BigDecimal;
import java.util.*;

import java.security.Principal;

public class LoginSessionFacadeBean implements SessionBean {
	
	private ClienteLocalHome clienteHome;
	private UsuarioLocalHome usuarioHome;
	
	ClienteDAO clienteDAO;
	
	private SessionContext ctx;
	
	public void ejbCreate() throws CreateException {
		System.out.println("LoginSessionFacadeBean: ejbCreate()");
		try {
			ServiceLocator locator = ServiceLocator.getInstance();
			clienteHome = (ClienteLocalHome) locator.findLocalEJB("ejb/loja/cliente");
			usuarioHome = (UsuarioLocalHome) locator.findLocalEJB("ejb/loja/usuario");
			clienteDAO = JdbcClienteDAO.getInstance();
		} catch (LojaException e) {
			throw new CreateException("Service Locator failure: " + e);
		}
	}
	
	/**
	 * Versao nao-JAAS do login.
	 */
	public boolean simpleLogin(String userid, String senha) {
		System.out.println("AdminLojaSessionFacadeBean: login("+ userid+", senha)");
		try {
			UsuarioLocal usuario = usuarioHome.findByPrimaryKey(new UsuarioPK(userid));
			String senhaReal = usuario.getSenha();
			if (senhaReal.equals(senha)) {
				return true;
			}
			return false;
		} catch (FinderException e) {
			System.out.println("Usuario " + userid + " nao existe!");
			return false;
		} catch (Exception e) {
			return false;
		}
	}
	
	public void cadastrar(ClienteVO clienteVO) throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: cadastrar("+ clienteVO+")");
		try {
			String userid = clienteVO.getUserid();
			String senha = clienteVO.getSenha();
			String grupo = clienteVO.getGrupo();
			String nome = clienteVO.getNome();
			String email = clienteVO.getEmail();
			String endereco = clienteVO.getEndereco();
			String cartao = clienteVO.getCartao();
			clienteHome.create(userid, senha, grupo, nome, email, endereco, cartao);
		} catch (CreateException e) {
			throw new LojaException(e);
		}
	}
	
	public void atualizar(ClienteVO clienteVO) throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: atualizar("+ clienteVO+")");
		try {
			String userid = clienteVO.getUserid();
			ClienteLocal clienteBean = clienteHome.findByPrimaryKey(new ClientePK(userid));
			clienteBean.setSenha(clienteVO.getSenha());
			clienteBean.setGrupo(clienteVO.getGrupo());
			clienteBean.setNome(clienteVO.getNome());
			clienteBean.setEmail(clienteVO.getEmail());
			clienteBean.setEndereco(clienteVO.getEndereco());
			clienteBean.setCartao(clienteVO.getCartao());
		} catch (FinderException e) {
			throw new LojaException(e);
		}
	}
	
	public boolean existe(String userid) throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: existe("+ userid+")");
		return !clienteDAO.isUseridFree(userid);
	}
	
	public ClienteVO mostrar(String userid) throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: mostrar("+ userid+")");
		
		// Verificar quem estah logado
		String loggedUser = ctx.getCallerPrincipal().getName();
		System.out.println(loggedUser + " esta usando o metodo AdminLojaSessionFacadeBean.mostrar()");
		
		// Se usuario nao eh adiministrador, so pode ver dados se ele for o usuario pesquisado!
		if( !ctx.isCallerInRole("admin")) {
			// nao eh administrador
			if (!loggedUser.equals(userid)) {
				throw new LojaException("Usuario " + loggedUser + " estah tentando ver os dados de " + userid + "!");
			}
		}
		
		ClienteVO clienteVO = null;
		try {
			ClienteLocal clienteBean = clienteHome.findByPrimaryKey(new ClientePK(userid));
			String senha = clienteBean.getSenha();
			String grupo = clienteBean.getGrupo();
			String nome = clienteBean.getNome();
			String email = clienteBean.getEmail();
			String endereco = clienteBean.getEndereco();
			String cartao = clienteBean.getCartao();
			return new ClienteVO(userid, senha, grupo, nome, email, endereco, cartao);
		} catch (FinderException e) {
			throw new LojaException(e);
		}
	}
	
	/**
	 * Teste
	 */
	public String getCurrentUserid() throws LojaException {
		Principal principal = ctx.getCallerPrincipal();
		System.out.print("Security principal name: " + principal.getName());
		return principal.getName();
	}
	
	public void ejbActivate() {}
	public void ejbPassivate() {}
	public void ejbRemove() {}
	public void setSessionContext(SessionContext ctx) {
		this.ctx = ctx;
	}
}
