package loja.ejb;

import loja.integration.*;
import loja.*;

import javax.ejb.*;
import javax.jms.*;
import java.math.BigDecimal;
import java.util.*;

public class LojaSessionFacadeBean implements SessionBean {
	
	/**
	 * Fila do JBoss usada quando esta aplicao for chamada por
	 * clientes que nao usam o ENC (java:comp/env)
	 */
	private static final String FALLBACK_QUEUE_NAME = "queue/B"; // (pedidos)
	
	private ClienteLocalHome clienteHome;
	private CompraLocalHome compraHome;
	private ItemLocalHome itemHome;
	private ProdutoLocalHome produtoHome;
	
	private CarrinhoSession carrinho;
	private CompraLocal compra;
	private ClienteLocal cliente;
	private SessionContext ctx;
	
	private Queue filaPedidos;
	private QueueConnection jmsCon;
	
	public void ejbCreate() throws CreateException {
		System.out.println("LojaSessionFacadeBean: ejbCreate()");
		try {
			ServiceLocator locator = ServiceLocator.getInstance();
			produtoHome = (ProdutoLocalHome) locator.findLocalEJB("ejb/loja/produto");
		} catch (LojaException e) {
			throw new CreateException("Service Locator failure: " + e);
		}
	}

	public CarrinhoSession getCarrinho() throws LojaException {
		System.out.println("LojaSessionFacadeBean: getCarrinho()");
		if (carrinho == null) {
			criarCarrinho();
		}
		return carrinho;
	}
	
	public void criarCarrinho() throws LojaException {
		System.out.println("LojaSessionFacadeBean: criarCarrinho()");
		try {
			ServiceLocator locator = ServiceLocator.getInstance();
			CarrinhoSessionHome carrinhoHome = (CarrinhoSessionHome)locator.findRemoteEJB("ejb/loja/carrinho", CarrinhoSessionHome.class);
			carrinho = carrinhoHome.create();
		} catch (CreateException e) {
			throw new LojaException("LojaSessionFacadeBean - criarCarrinho(): " + e);
		} catch (java.rmi.RemoteException e) {
			throw new LojaException("LojaSessionFacadeBean - criarCarrinho(): " + e);
		}
	}
	
	public ProdutoVO[] listarProdutosDisponiveis() throws LojaException {
		System.out.println("LojaSessionFacadeBean: listarProdutosDisponiveis()");
		System.out.println("AdminLojaSessionFacadeBean: listarProdutos()");
		try {
			Collection beans = produtoHome.findAll();
			System.out.println("AdminLojaSessionFacadeBean: listarProdutos():Achados " + beans.size() + " beans.");
			Iterator it = beans.iterator();
			List produtos = new ArrayList();
			while(it.hasNext()) {
				ProdutoLocal produtoBean = (ProdutoLocal) it.next();
				int quantidade = produtoBean.getQuantidade();
				if (quantidade > 0) {
					ProdutoVO produtoVO = new ProdutoVO();
					produtoVO.setCodigo(produtoBean.getCodigo());
					produtoVO.setNome(produtoBean.getNome());
					produtoVO.setPreco(produtoBean.getPreco());
					produtoVO.setQuantidade(quantidade);
					produtos.add(produtoVO);
				}
			}
			ProdutoVO[] buffer = new ProdutoVO[produtos.size()];
			return (ProdutoVO[]) produtos.toArray(buffer);
		} catch (FinderException e) {
			throw new LojaException(e);
		}
	}

	public CompraVO processarCompra(String userid, ItemVO[] carrinho) throws LojaException {
		System.out.println("LojaSessionFacadeBean: processarCompra("+userid+", "+carrinho+")");
		String mensagem = "";
		if (carrinho.length <= 0) {
			throw new LojaException("Carrinho de compras estah vazio!");
		}
		try {
			ServiceLocator locator = ServiceLocator.getInstance();
			clienteHome = (ClienteLocalHome) locator.findLocalEJB("ejb/loja/cliente");
			cliente = clienteHome.findByPrimaryKey(new ClientePK(userid));
			
			CompraDAO dao = JdbcCompraDAO.getInstance();
			ItemDAO itemDAO = JdbcItemDAO.getInstance();
			
			String numeroNota = dao.criarNumero();
			compraHome = (CompraLocalHome) locator.findLocalEJB("ejb/loja/compra");
			itemHome = (ItemLocalHome) locator.findLocalEJB("ejb/loja/item");
			compra = compraHome.create(numeroNota, cliente.getUserid());
			
			for (int i = 0; i < carrinho.length; i++) {
				ProdutoVO produtoVO = carrinho[i].getProduto();
				ProdutoLocal produto = produtoHome.findByPrimaryKey(new ProdutoPK(produtoVO.getCodigo()));
				int qte = carrinho[i].getQuantidade();
				
				// Controle previo de estoque
				int estoque = produto.getQuantidade();
				if (estoque == 0) {
					mensagem += "\nO produto [" + produto.getCodigo() + "] " + produto.getNome() + " estah esgotado!";
					continue; // nao acrescentar produto na nota
				} else if (qte > estoque) {
					qte = estoque;
					mensagem += "\nO produto [" + produto.getCodigo() + "] " + produto.getNome() + " estah com estoque limitado a " + estoque + " unidades! O seu pedido foi reduzido para este valor.";
				}

				ItemLocal item = itemHome.create(itemDAO.criarNumero(), produto, compra.getNumero(), qte);
				compra.addItem(item);
			}
			return gerarCompraVO();
		} catch (FinderException e) {
			throw new LojaException("LojaSessionFacadeBean - processarCompra(): " + e);
		} catch (CreateException e) {
			throw new LojaException("LojaSessionFacadeBean - processarCompra(): " + e);
		} 
	}
	
	public String comprar() throws LojaException {
		System.out.println("LojaSessionFacadeBean: comprar()");
		// Realizar compra
		encaminharPedido(compra);
		compra.setStatus(Status.EM_ANDAMENTO);
		// 2. Enviar mensagem (opcional)
		String mensagem = "Validacao de cartao estah pendente. Aguarde um e-mail com a confirmacao de seu pedido.";
		return mensagem;
	}
	
	private void encaminharPedido(CompraLocal compra) throws LojaException {
		System.out.println("LojaSessionFacadeBean: encaminharPedido("+compra+")");
		try {
			ServiceLocator locator = ServiceLocator.getInstance();
			jmsCon = locator.findDefaultJMSQueueConnection();
			Queue filaPedidos = locator.findQueue("jms/loja/pedidos", FALLBACK_QUEUE_NAME);
			
			QueueSession session = jmsCon.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
			Message message = session.createMessage();
			message.setStringProperty("Compra", compra.getNumero());
			QueueSender sender = session.createSender( filaPedidos );
			sender.send(message);
		} catch (LojaException e) {
			throw new LojaException("Service Locator failure: " + e);
		} catch (javax.jms.JMSException e) {
			throw new LojaException(e);
		}
	}
	
	private CompraVO gerarCompraVO() {
		ItemLocal[] itens = compra.getItens();
		ItemVO[] itensVO = new ItemVO[itens.length];
		for (int i = 0; i < itens.length; i++) {
			String numero = itens[i].getNumero();
			ProdutoLocal p = itens[i].getProduto();
			ProdutoVO produtoVO = new ProdutoVO(p.getCodigo(), p.getNome(), p.getPreco(), p.getQuantidade());
			int quantidade = itens[i].getQuantidade();
			itensVO[i] = new ItemVO(numero, produtoVO, compra.getNumero(), quantidade);
		}
		CompraVO compraVO = new CompraVO(compra.getNumero(), compra.getClienteId(), compra.getStatus());
		compraVO.setItens(itensVO);
		return compraVO;
	}
	
	public void ejbActivate() {}
	public void ejbPassivate() {}
	public void ejbRemove() {}
	public void setSessionContext(SessionContext ctx) {
		this.ctx = ctx;
	}
}
