package loja.ejb;

import javax.ejb.*;
import javax.jms.*;
import java.util.*;

import javax.mail.Transport;
import javax.mail.internet.*;

import loja.integration.ServiceLocator;
import loja.LojaException;

/**
 * Usa a API JavaMail para enviar e-mails cujos dados so depositados
 * em uma fila JMS.
 */
public class MailerMDB implements MessageDrivenBean, MessageListener {
	
	private MessageDrivenContext ctx;
	private String mailServer;
	
	public void ejbCreate() {
		System.out.println("MailerMDB: ejbCreate()");
		try {
			ServiceLocator locator = ServiceLocator.getInstance();
			mailServer = (String) locator.findEnvEntry("email/server");
		} catch (LojaException e) {
			System.out.println("MailerMDB-ejbCreate(): " + e);
		}
	}
	
	public void ejbRemove() {}
	public void setMessageDrivenContext(MessageDrivenContext ctx) {
		System.out.println("MailerMDB: setMessageDrivenContext()");
		this.ctx = ctx;
	}
	
	public void onMessage(Message message) {
		System.out.println("MailerMDB: onMessage("+message+")");
		try {
			String texto = "[Vazio]";
			if (message instanceof TextMessage) {
				TextMessage textMessage = (TextMessage) message;
				texto = textMessage.getText();
			}
			String assunto = message.getStringProperty("Assunto");
			String remetente = message.getStringProperty("De");
			String destinatario = message.getStringProperty("Para");
			
			sendMail(remetente, destinatario, assunto, texto);
			
		} catch (JMSException e) {
			System.out.println("MailerMDB-onMessage(): " + e);
		}
	}
	
	/**
	 * Envia e-mail usando JavaMail.
	 */
	private void sendMail(String remetente, String destinatario, String assunto, String mensagem) {
        java.util.Properties props = new java.util.Properties();
		props.setProperty("mail.smtp.host", mailServer);
        javax.mail.Session session = javax.mail.Session.getDefaultInstance(props, null);
        
        try {
            javax.mail.Message msg = new MimeMessage(session);
            msg.setFrom(new InternetAddress(remetente));
            msg.setRecipients(javax.mail.Message.RecipientType.TO, InternetAddress.parse(destinatario, false));
            msg.setSubject(assunto);
            msg.setText(mensagem);
            Transport.send(msg);
        } catch (AddressException e) {
            System.out.println("E-mail nao pode ser enviado pois endereco internet eh invalido: " + e);
        } catch (javax.mail.MessagingException e) {
            System.out.println("E-mail nao pode ser enviado: " + e);
        }
    }
}
