package loja.ejb;

import javax.ejb.*;
import java.util.*;

import loja.integration.*;
import loja.*;

public class UsuarioBean implements EntityBean {
	
	protected EntityContext ctx;
	
	private String userid;
	private String senha;
	private String grupo;
	
	private UsuarioDAO usuarioDAO;
	
	//////////////////// life-cycle ////////////////////////////

	/////////// 1. HOME ///////////////

	public void  ejbPostCreate(String userid, String senha) {}
	public UsuarioPK ejbCreate(String userid, String senha) throws CreateException, UsuarioJaExisteException {
		return ejbCreate(userid, senha, "todos");
	}
	
	public void  ejbPostCreate(String userid, String senha, String grupo) {}
	public UsuarioPK ejbCreate(String userid, String senha, String grupo) throws CreateException, UsuarioJaExisteException {
		System.out.println("UsuarioBean: ejbCreate("+userid+","+ senha+","+ grupo+")");
		this.userid = userid;
		this.senha = senha;
		this.grupo = grupo;
		try {
			usuarioDAO = JdbcUsuarioDAO.getInstance();
			if (!usuarioDAO.isUseridFree(userid)) {
				throw new UsuarioJaExisteException("Usuario " + userid + " ja existe. Escolha outro!");	
			}
			UsuarioVO usuario = new UsuarioVO(userid, senha, grupo);
			return new UsuarioPK(usuarioDAO.create(usuario));
		} catch (UsuarioJaExisteException e) {
			throw e;
		} catch (LojaException e) {
			throw new CreateException("UsuarioBean: Incapaz de obter instancia DAO - ejbCreate(): " + e);
		}
	}

	public UsuarioPK ejbFindByPrimaryKey(UsuarioPK pk) throws FinderException {
		System.out.println("UsuarioBean: ejbFindByPrimaryKey()");
		try {
			usuarioDAO = JdbcUsuarioDAO.getInstance();
			if (usuarioDAO.exists(pk.userid)) {
				return pk;
			} else {
				throw new ObjectNotFoundException("Objeto " + pk.userid + " nao existe!");
			}
		} catch (LojaException e) {
			throw new EJBException("UsuarioBean: Falha na comunicacao com o banco - ejbFindByPrimaryKey(): " + e);
		}
	}
	
	public Collection ejbFindAll() throws FinderException {
		System.out.println("UsuarioBean: ejbFindAll()");
		String[] userids = null;
		try {
			usuarioDAO = JdbcUsuarioDAO.getInstance();
			userids = usuarioDAO.getAll();
		} catch (LojaException e) {
			throw new EJBException("UsuarioBean: Falha na comunicacao com o banco - ejbFindAll(): " + e);
		}
		List list = new ArrayList(userids.length);
		for (int i = 0; i < userids.length; i++) {
			list.add(new UsuarioPK(userids[i]));
		}
		return list;
	}
	
	/////////// 2. CONTAINER CALL-BACK ///////////////////
	public void ejbLoad() {
		System.out.println("UsuarioBean: ejbLoad()");
		UsuarioPK pk = (UsuarioPK)ctx.getPrimaryKey(); 
		UsuarioVO usuario = null;
		try {
			usuarioDAO = JdbcUsuarioDAO.getInstance();
			usuario = usuarioDAO.load(pk.userid);
		} catch (LojaException e) {
			throw new EJBException("UsuarioBean: Falha na comunicacao com o banco - ejbLoad(): " + e);
		}
		userid = usuario.getUserid();
		senha   = usuario.getSenha();
		grupo  = usuario.getGrupo();
	}
	
	public void ejbStore() {
		System.out.println("UsuarioBean: ejbStore()");
		UsuarioVO usuario = new UsuarioVO(userid, senha, grupo);
		try {
			usuarioDAO = JdbcUsuarioDAO.getInstance();
			usuarioDAO.store(usuario);
		} catch (LojaException e) {
			throw new EJBException("Falha na comunicacao com o banco - ejbStore(): " + e);
		}
	}
	
	public void ejbRemove() {
		System.out.println("UsuarioBean: ejbRemove()");
		UsuarioPK pk = (UsuarioPK)ctx.getPrimaryKey();
		try {
			usuarioDAO = JdbcUsuarioDAO.getInstance();
			usuarioDAO.remove(pk.userid);
		} catch (LojaException e) {
			throw new EJBException("UsuarioBean: Falha na comunicacao com o banco - ejbRemove(): " + e);
		}
	}
	
	public void setEntityContext(EntityContext ctx) {
		System.out.println("UsuarioBean: setEntityContext()");
		this.ctx = ctx;
	}
	
	public void unsetEntityContext() {
		ctx = null;
	}
	
	public void ejbActivate() {}
	public void ejbPassivate() {}
	
	
	//////////// BUSINESS AND ACESSOR ///////////////
	
	public String getUserid() {
		System.out.println("UsuarioBean: getUserid()");
		return userid;
	}
	public void setUserid(String userid) {
		System.out.println("UsuarioBean: setUserId("+userid+")");
		this.userid = userid;
	}
	
	public String getSenha() {
		System.out.println("UsuarioBean: getSenha()");
		return senha;
	}
	public void setSenha(String senha) {
		System.out.println("UsuarioBean: setSenha("+senha+")");
		this.senha = senha;
	}
	
	public String getGrupo() {
		System.out.println("UsuarioBean: getGrupo()");
		return grupo;
	}
	public void setGrupo(String grupo) {
		System.out.println("UsuarioBean: setGrupo("+grupo+")");
		this.grupo = grupo;
	}

}
