package loja.integration;

import loja.*;

import java.sql.*;
import java.util.*;
import java.math.BigDecimal;

/**
create table clientes (
	id varchar(8) constraint pk_clientes primary key,
	nome varchar(64),
	email varchar(64),
	endereco varchar(256),
	cartao varchar(16),
	constraint fk_usuario foreign key (id) references usuarios(id)
);
*/
public class JdbcClienteDAO implements ClienteDAO {
	
	/** SQL queries */
	public static final String SQL_USERID = "select id from clientes where id = ?";
	public static final String SQL_CREATE = "insert into clientes(id, nome, email, endereco, cartao) values (?, ?, ?, ?, ?)";
	public static final String SQL_REMOVE = "delete from clientes where id = ?";
	public static final String SQL_LOAD   = "select id, nome, email, endereco, cartao from clientes where id = ?";
	public static final String SQL_STORE  = "update clientes set nome = ?, email = ?, endereco = ?, cartao = ? where id = ?";
	public static final String SQL_PK     = "select id from clientes where id = ?";
	public static final String SQL_ALL    = "select id from clientes";
	public static final String SQL_NOME   = "select id from clientes where nome = ?";
	
	private static ClienteDAO instance = null;
	private ServiceLocator locator;
	
	private JdbcClienteDAO() throws LojaException {
		locator = ServiceLocator.getInstance();
	}
	
	private Connection getConnection() throws LojaException {
		Connection con = locator.findJdbcConnection("jdbc/LojaDB");
		return con;
	}
	
	public static ClienteDAO getInstance()  throws LojaException {
		if (instance == null) {
			instance = new JdbcClienteDAO();
		}
		return instance;
	}
	
	public boolean isUseridFree(String userid) throws LojaException {
		String sql = SQL_USERID;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setString(1, userid);
			ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				return false;
			} else {
				return true;
			}
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
	
	public String create(ClienteVO cliente) throws LojaException {
		String sql = SQL_CREATE;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		String userid = cliente.getUserid();
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setString(1, userid);
			stmt.setString(2, cliente.getNome());
			stmt.setString(3, cliente.getEmail());
			stmt.setString(4, cliente.getEndereco());
			stmt.setString(5, cliente.getCartao());
			stmt.executeUpdate();
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
		return userid;
	}
	
	public void remove(String userid) throws LojaException {
		String sql = SQL_REMOVE;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setString(1, userid);
			stmt.executeUpdate();
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
	
	public ClienteVO load(String userid) throws LojaException  {
		String sql = SQL_LOAD;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		ClienteVO cliente = new ClienteVO();
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setString(1, userid);
			ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				cliente.setUserid(rs.getString(1));
				cliente.setNome(rs.getString(2));
				cliente.setEmail(rs.getString(3));
				cliente.setEndereco(rs.getString(4));
				cliente.setCartao(rs.getString(5));
			}
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
		return cliente;
	}
	
	public void store(ClienteVO cliente) throws LojaException {
		String sql = SQL_STORE;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setString(1, cliente.getNome());
			stmt.setString(2, cliente.getEmail());
			stmt.setString(3, cliente.getEndereco());
			stmt.setString(4, cliente.getCartao());
			stmt.setString(5, cliente.getUserid());
			stmt.executeUpdate();
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
	
	public boolean exists(String userid) throws LojaException {
		String sql = SQL_PK;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		String useridLido = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setString(1, userid);
			ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				useridLido = rs.getString(1);
			}
			if (useridLido == null) {
				throw new SQLException("Cliente " + userid + " inexistente!");
			}
			return true;
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) { return false; }
		}
	}
	
	public String[] getAll() throws LojaException {
		String sql = SQL_ALL;
		System.out.println("Executando " + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		List useridList = new ArrayList();
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			ResultSet rs = stmt.executeQuery();
			while (rs.next()) {
				useridList.add(rs.getString(1));
			}
			String[] buffer = new String[useridList.size()];
			return (String[]) useridList.toArray(buffer);
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
	
	public String[] getByNome(String nome) throws LojaException {
		String sql = SQL_NOME;
		System.out.println("Executando " + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		List useridList = new ArrayList();
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setString(1, nome);
			ResultSet rs = stmt.executeQuery();
			while (rs.next()) {
				useridList.add(rs.getString(1));
			}
			String[] buffer = new String[useridList.size()];
			return (String[]) useridList.toArray(buffer);
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
}
