package loja.integration;

import loja.*;

import java.sql.*;
import java.util.*;

/**
create table itens (
	id integer constraint pk_itens primary key,
	compra integer,
	produto integer,
	qte integer,
	constraint fk_compra foreign key (compra) references compras(id),
	constraint fk_produto foreign key (produto) references produtos(id)
);
*/


///////// TO FIX - LOAD - ///////////////
public class JdbcItemDAO implements ItemDAO {
	
	/** SQL queries */
	public static final String SQL_NUMERO  = "select max(id) from itens";
	public static final String SQL_CREATE  = "insert into itens(id, compra, produto, qte) values (?, ?, ?, ?)";
	public static final String SQL_REMOVE  = "delete from itens where id = ?";
	public static final String SQL_LOAD    = "select id, compra, produto, qte from itens where id = ?";
	public static final String SQL_STORE   = "update itens set compra = ?, produto = ?, qte = ? where id = ?";
	public static final String SQL_PK      = "select id from itens where id = ?";
	public static final String SQL_ALL     = "select id from itens";
	public static final String SQL_COMPRA  = "select id from itens where compra = ?";
	public static final String SQL_PRODUTO = "select id from itens where produto = ?";
	
	private static ItemDAO instance = null;
	private ServiceLocator locator;
	
	private JdbcItemDAO() throws LojaException {
		locator = ServiceLocator.getInstance();
	}
	
	private Connection getConnection() throws LojaException {
		Connection con = locator.findJdbcConnection("jdbc/LojaDB");
		return con;
	}
	
	public static ItemDAO getInstance()  throws LojaException {
		if (instance == null) {
			instance = new JdbcItemDAO();
		}
		return instance;
	}
	
	public String criarNumero() throws LojaException {
		String sql = SQL_NUMERO;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		String numero = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				int ultimo = rs.getInt(1);
				numero = "" + (ultimo + 1);
			} else {
				numero = "1";
			}
			if (numero == null) {
				throw new SQLException("Incapaz de criar novo numero!");
			}
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
		return numero;
	}
	
	public String create(ItemVO item) throws LojaException {
		String sql = SQL_CREATE;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		String numero = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			numero = item.getNumero();
			stmt.setInt(1, Integer.parseInt(numero));
			stmt.setInt(2, Integer.parseInt(item.getCompraId()));
			stmt.setString(3, item.getProdutoId());
			stmt.setInt(4, item.getQuantidade());
			stmt.executeUpdate();
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close();
		    } catch (SQLException e) {}
		}
		return numero;
	}
	
	public void remove(String numero) throws LojaException {
		String sql = SQL_REMOVE;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setInt(1, Integer.parseInt(numero));
			stmt.executeUpdate();
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
	
	public ItemVO load(String numero) throws LojaException  {
		String sql = SQL_LOAD;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		ItemVO item = new ItemVO();
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setInt(1, Integer.parseInt(numero));
			ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				item.setNumero(""+rs.getInt(1));
				item.setCompraId(""+rs.getInt(2));
				item.setProdutoId(rs.getString(3));
				item.setQuantidade(rs.getInt(4)); 
			}
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
		return item;
	}
	
	public void store(ItemVO item) throws LojaException {
		String sql = SQL_STORE;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setInt(1, Integer.parseInt(item.getCompraId()));
			stmt.setString(2, item.getProdutoId());
			stmt.setInt(3, item.getQuantidade());
			stmt.setInt(4, Integer.parseInt(item.getNumero()));
			stmt.executeUpdate();
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
	
	public boolean exists(String numero) throws LojaException {
		String sql = SQL_PK;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		int numeroLido = -1;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setInt(1, Integer.parseInt(numero));
			ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				numeroLido = rs.getInt(1);
			}
			if (numeroLido == -1) {
				throw new SQLException("Registro " + numero + " inexistente!");
			}
			return true;
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) { return false; }
		}
	}
	
	public String[] getAll() throws LojaException {
		String sql = SQL_ALL;
		System.out.println("Executando " + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		List numeroList = new ArrayList();
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			ResultSet rs = stmt.executeQuery();
			while (rs.next()) {
				numeroList.add(""+rs.getInt(1));
			}
			String[] buffer = new String[numeroList.size()];
			return (String[]) numeroList.toArray(buffer);
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
	
	public String[] getByCompra(String compraId) throws LojaException {
		String sql = SQL_COMPRA;
		System.out.println("Executando " + sql);
		Connection con = null;PreparedStatement stmt = null;
		List numeroList = new ArrayList();
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setInt(1, Integer.parseInt(compraId));
			ResultSet rs = stmt.executeQuery();
			while (rs.next()) {
				numeroList.add(""+rs.getInt(1));
			}
			String[] buffer = new String[numeroList.size()];
			return (String[]) numeroList.toArray(buffer);
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
	
	public String[] getByProduto(String produtoId) throws LojaException {
		String sql = SQL_PRODUTO;
		System.out.println("Executando " + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		List numeroList = new ArrayList();
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setInt(1, Integer.parseInt(produtoId));
			ResultSet rs = stmt.executeQuery();
			while (rs.next()) {
				numeroList.add(""+rs.getInt(1));
			}
			String[] buffer = new String[numeroList.size()];
			return (String[]) numeroList.toArray(buffer);
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
}
