package loja.integration;

import loja.*;

import java.sql.*;
import java.util.*;
import java.math.BigDecimal;

/**
create table usuarios (
	id varchar(8) constraint pk_usuarios primary key,
	senha varchar(16),
	grupo varchar(16)
);
*/
public class JdbcUsuarioDAO implements UsuarioDAO {
	
	/** SQL queries */
	public static final String SQL_USERID = "select id from usuarios where id = ?";
	public static final String SQL_CREATE = "insert into usuarios(id, senha, grupo) values (?, ?, ?)";
	public static final String SQL_REMOVE = "delete from usuarios where id = ?";
	public static final String SQL_LOAD   = "select id, senha, grupo from usuarios where id = ?";
	public static final String SQL_STORE  = "update usuarios set senha = ?, grupo = ? where id = ?";
	public static final String SQL_PK     = "select id from usuarios where id = ?";
	public static final String SQL_ALL    = "select id from usuarios";
	
	private static UsuarioDAO instance = null;
	private ServiceLocator locator;
	
	private JdbcUsuarioDAO() throws LojaException {
		locator = ServiceLocator.getInstance();
	}
	
	private Connection getConnection() throws LojaException {
		Connection con = locator.findJdbcConnection("jdbc/LojaDB");
		return con;
	}
	
	public static UsuarioDAO getInstance()  throws LojaException {
		if (instance == null) {
			instance = new JdbcUsuarioDAO();
		}
		return instance;
	}
	
	public boolean isUseridFree(String userid) throws LojaException {
		String sql = SQL_USERID;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setString(1, userid);
			ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				return false;
			} else {
				return true;
			}
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
	
	public String create(UsuarioVO usuario) throws LojaException {
		String sql = SQL_CREATE;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		String userid = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setString(1, usuario.getUserid());
			stmt.setString(2, usuario.getSenha());
			stmt.setString(3, usuario.getGrupo());
			stmt.executeUpdate();
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
		return userid;
	}
	
	public void remove(String userid) throws LojaException {
		String sql = SQL_REMOVE;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setString(1, userid);
			stmt.executeUpdate();
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
	
	public UsuarioVO load(String userid) throws LojaException  {
		String sql = SQL_LOAD;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		UsuarioVO usuario = new UsuarioVO();
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setString(1, userid);
			ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				usuario.setUserid(rs.getString(1));
				usuario.setSenha(rs.getString(2));
				usuario.setGrupo(rs.getString(3));
			}
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
		return usuario;
	}
	
	public void store(UsuarioVO usuario) throws LojaException {
		String sql = SQL_STORE;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setString(1, usuario.getSenha());
			stmt.setString(2, usuario.getGrupo());
			stmt.setString(3, usuario.getUserid());
			stmt.executeUpdate();
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
	
	public boolean exists(String userid) throws LojaException {
		String sql = SQL_PK;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		String useridLido = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setString(1, userid);
			ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				useridLido = rs.getString(1);
			}
			if (useridLido == null) {
				return false;
			}
			return true;
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) { return false; }
		}
	}
	
	public String[] getAll() throws LojaException {
		String sql = SQL_ALL;
		System.out.println("Executando " + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		List useridList = new ArrayList();
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			ResultSet rs = stmt.executeQuery();
			while (rs.next()) {
				useridList.add(rs.getString(1));
			}
			String[] buffer = new String[useridList.size()];
			return (String[]) useridList.toArray(buffer);
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
}
