package loja.web; 

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.InputStream;

import loja.*;

/**
 * $Revision: 1.2 $ $Date: 2003/05/25 15:53:02 $ (helder@acm.org).
 * Esta classe  usada como base para todos os servlets da aplicao que
 * precisam apemas implementar um mtodo.
 */
public abstract class BaseServlet extends HttpServlet {

    private CommandFactory commandFactory;
	private loja.BusinessDelegate delegate;
	
	/**
	 * Mtodo que precisa ser implementado em cada servlet retornando uma instancia
	 * de um BusinessDelegate a ser passado aos objetos de comando via execute().
	 */
	protected abstract loja.BusinessDelegate getBusinessDelegate() throws LojaException;

	/**
	 * Na inicializao, carrega o parametro de contexto que informa a localizao do arquivo
	 * que contm os mapeamentos entre nomes de comandos e classes que os implementam. Obtm
	 * um stream para este arquivo e utiliza-o para inicializar o CommandFactory, que permitir
	 * a obteno de um objeto Command, atravs do nome.
	 */
    public void init() throws UnavailableException {
        String mappingsFile = getServletContext().getInitParameter("mappings-file");
        if (mappingsFile == null) {
            throw new UnavailableException("Required parameter mappings-file not found as <context-param> in deployment descriptor");
        }
		try {
			delegate = getBusinessDelegate();
		} catch (LojaException e) {
			throw new UnavailableException("Falha na inicializao do Business Delegate: "+e);
		}
        
        // This class receives the name of an XML file which contains command-action object mappings
        // and returns an object that can return action objects by name.
        try {
            InputStream stream = getServletContext().getResourceAsStream(mappingsFile);
            commandFactory = CommandFactory.getInstance(stream);
        } catch (Exception e) {
            throw new UnavailableException("Falha na inicializao: "+e);
        }
    }
	
	/**
	 * Simplesmente redireciona para process().
	 */
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, java.io.IOException {
		process(request, response);
	}
	/**
	 * Simplesmente redireciona para process().
	 */
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, java.io.IOException {
		process(request, response);
	}

	/**
	 * L parmetro "cmd", se existir e produz um objeto comando correspondente. 
	 * Executa o objeto comando, chamando seu mtodo execute() e passando request, response e
	 * uma instncia do business delegate usado (ou null, se ele no existir). O mtodo execute()
	 * deve retornar uma String contendo a URL da pgina JSP ou servlet a quem a requisio 
	 * repassada via RequestDispatcher.
	 */
    public void process(HttpServletRequest request, HttpServletResponse response) throws ServletException, java.io.IOException {

        Command command = null;
        String commandName = request.getParameter("cmd");
        if (commandName == null) {
            command = commandFactory.getCommand("servlet");
        } else {
            try {
                command = commandFactory.getCommand(commandName);
            } catch (CommandNotFoundException e) {
                command = commandFactory.getCommand("default");
            }
        }

        Object result = command.execute(request, response, delegate);
        if (result instanceof String) { // deve conter url da proxima pagina
            RequestDispatcher dispatcher = request.getRequestDispatcher((String)result);
            dispatcher.forward(request, response);
        } else {
            throw new ServletException("Command returned an invalid response: " + result);
        }
    }
}
