package loja.web;

import java.util.HashMap;
import java.io.*;
import javax.xml.parsers.*;

/**
 * This class returns an implementation of an action command object 
 * by name. Action commands are delegates assigned by the controller
 * servlet to process specific commands issued by the presentation layer.
 * It uses the SAX API to parse the XML file which contains the action
 * mappings via the CommandFactorySAXHandler class.
 * $Revision: 1.1.1.1 $ $Date: 2003/05/23 16:16:36 $ (helder@acm.org).
 */
public class CommandFactory {
    
    /**
     * Collection of command action objects.
     */
    private HashMap commands = new HashMap();
    
    /////////////// Singleton implementation /////////////////////////////////////
    /**
     * Singleton implementation.
     */
    private static CommandFactory instance = null;
    private CommandFactory(InputStream stream) throws IOException {
        try {
            SAXParserFactory saxFactory = SAXParserFactory.newInstance();
            SAXParser parser = saxFactory.newSAXParser();
            parser.parse(stream, new CommandFactorySAXHandler(commands)); // HashMap is populated by this class
        } catch (Exception e) {
            if (e instanceof RuntimeException) {
                throw (RuntimeException) e;
            } else {
                throw new IOException("CommandFactory - Unable to initialize command table. " + e);
            }
        }
    }
    
    /**
     * Returns an instance of this factory.
     */
    public static CommandFactory getInstance(InputStream mappings) throws IOException {
        if (instance == null) {
            return new CommandFactory(mappings);
        }
        return instance;
    }
    /////////////// End Singleton implementation //////////////

    
    /**
	 * /////////////// Main instance utility method ///////////////
     * Returns a command action object mapped to a supplied command name.
     */
    public Command getCommand(String name) throws CommandNotFoundException {
        Command cmd = (Command)commands.get(name);
        if (cmd == null) {
            throw new CommandNotFoundException("No action object is bound to the command '" + name + "'");
        }
        return cmd;
    }
}
