package loja.web.command;
 
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;

import loja.*;
import loja.web.*;

/**
 * $Revision: 1.2 $ $Date: 2003/05/26 08:22:22 $ (helder@acm.org).
 */
public class AtualizarCarrinhoCommand implements Command {
    
    private String successUrl;
    private String failureUrl;
    
    public AtualizarCarrinhoCommand(String successUrl, String failureUrl) {
        this.successUrl = successUrl;
        this.failureUrl = failureUrl;
    }
    
    /**
     * Reads all messages from XML files.
     */
    public Object execute(HttpServletRequest request, HttpServletResponse response, BusinessDelegate delegate) throws CommandException {

		LojaBusinessDelegate loja = (LojaBusinessDelegate)delegate;
		
		// Extrair as quantidades de cada produto no carrinho
		HashMap quantidades = new HashMap(); // mapeia codigo do produto a sua quantidade no carrinho
		Enumeration parameters = request.getParameterNames();
		while(parameters.hasMoreElements()) {
			String parameterName = (String)parameters.nextElement();
			if (parameterName.startsWith("qte_")) { // quantidade de algum produto
				int inicio = parameterName.indexOf("_") + 1;
				String codigo = parameterName.substring(inicio, parameterName.length());
				String qteStr = request.getParameter(parameterName);
				int qte = 1;
				try {
					qte = Integer.parseInt(qteStr);
				} catch (NumberFormatException e) { /* qte sera 1 se quantidade invalida */ }
				Integer quantidade = new Integer(qte);
				quantidades.put(codigo, quantidade);
			}
		}
		// se alguma quantidade for zero, remove o produto (isto eh o comportamento default do metodo adicionar)
        try {
            Iterator it = quantidades.keySet().iterator();
			while(it.hasNext()) { // ideal seria criar um metodo mais eficiente (com ItemCarrinho, que eh um VO)
				String codigo = (String) it.next();
				int qte = ( (Integer)quantidades.get(codigo) ).intValue();
				loja.adicionarProdutosCarrinho(codigo, qte);
			}
            return successUrl;
        } catch (Exception e) {
            throw new CommandException(e);
        } 
    }

    /**
     * Object.toString() implementation.
     */
    public String toString() {
        return this.getClass().getName() + "(\"" + successUrl + "\", \"" + failureUrl + "\")";
    }
    
    /**
     * Object.equals() implementation.
     */
    public boolean equals(Object obj) {
        if ( !(obj instanceof AtualizarCarrinhoCommand) ) return false;
        AtualizarCarrinhoCommand cmd = (AtualizarCarrinhoCommand) obj;
        return successUrl.equals(cmd.successUrl) && failureUrl.equals(cmd.failureUrl);
    }
    
    /**
     * Object.hashCode() implementation.
     */
    public int hashCode() {
        return 37 * successUrl.hashCode() * failureUrl.hashCode();
    }
}
