package loja.web.command;
 
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;

import loja.*;
import loja.web.*;

/**
 * $Revision: 1.2 $ $Date: 2003/05/24 02:02:08 $ (helder@acm.org).
 */
public class CriarProdutoCommand implements Command {
    
    private String successUrl;
    private String failureUrl;
    
    public CriarProdutoCommand(String successUrl, String failureUrl) {
        this.successUrl = successUrl;
        this.failureUrl = failureUrl;
    }
    
    public Object execute(HttpServletRequest request, HttpServletResponse response, BusinessDelegate delegate) throws CommandException {

		AdminLojaBusinessDelegate loja = (AdminLojaBusinessDelegate)delegate;
        String nome = request.getParameter("nome");
		String preco = request.getParameter("preco");
		
		List erros = new ArrayList();
		if (nome == null || nome.trim().length() == 0) {
			erros.add("Nome no pode ser vazio!");
		}
		if (preco != null && preco.trim().length() != 0) {
			double precoTemp = -1.0;
			try {
				precoTemp = Double.parseDouble(preco);
			} catch (NumberFormatException e) {}
			if(precoTemp == -1.0) {
				erros.add("Preo invlido! Escolha um nmero decimal positivo.");
			}
		} else {
			erros.add("Preo invlido! Escolha um nmero decimal.");
		}
		if (erros.size() > 0) {
			request.setAttribute("erros", erros);
			return failureUrl;
		}
		
		String codigo = null;
		try {
			codigo = loja.criarProduto(nome, new java.math.BigDecimal(preco));
		} catch (LojaException e) {
			throw new CommandException(e);
		}
		
		ProdutoVO valueObject = new ProdutoVO(codigo, nome, new java.math.BigDecimal(preco), 1);
		request.setAttribute("produto", valueObject);
		return successUrl;
    }
    
    /**
     * Object.toString() implementation.
     */
    public String toString() {
        return this.getClass().getName() + "(\"" + successUrl + "\", \"" + failureUrl + "\")";
    }
    
    /**
     * Object.equals() implementation.
     */
    public boolean equals(Object obj) {
        if ( !(obj instanceof CriarProdutoCommand) ) return false;
        CriarProdutoCommand cmd = (CriarProdutoCommand) obj;
        return successUrl.equals(cmd.successUrl) && failureUrl.equals(cmd.failureUrl);
    }
    
    /**
     * Object.hashCode() implementation.
     */
    public int hashCode() {
        return 37 * successUrl.hashCode() * failureUrl.hashCode();
    }
}
