package loja.web.command;
 
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;

import loja.*;
import loja.web.*;

/**
 * Simple redirection.
 * $Revision: 1.2 $ $Date: 2003/05/25 21:06:06 $ (helder@acm.org).
 */
public class GravarClienteCommand implements Command {
    
    private String successUrl;
    private String failureUrl;
    
    public GravarClienteCommand(String successUrl, String failureUrl) {
        this.successUrl = successUrl;
        this.failureUrl = failureUrl;
    }
    
    public Object execute(HttpServletRequest request, HttpServletResponse response, BusinessDelegate delegate) throws CommandException {

		LoginBusinessDelegate login = (LoginBusinessDelegate)delegate;
		String cmd = request.getParameter("cmd");
		
		ClienteVO cliente = new ClienteVO("","","","","","","");
		
        String param = request.getParameter("userid");
		if (param != null) 
			cliente.setUserid(param.trim());
		param = request.getParameter("nome");
		if (param != null) 
			cliente.setNome(param.trim());
		param = request.getParameter("email");
		if (param != null) 
			cliente.setEmail(param.trim());
		param = request.getParameter("endereco");
		if (param != null) 
			cliente.setEndereco(param.trim());
		param = request.getParameter("senha");
		if (param != null) 
			cliente.setSenha(param.trim());
		param = request.getParameter("grupo");
		if (param != null) 
			cliente.setGrupo(param.trim());
		param = request.getParameter("cartao");
		if (param != null) 
			cliente.setCartao(param.trim());
		
		List erros = new ArrayList();
		boolean dadosValidos = false;
		
		try {
			HttpSession session = request.getSession();
			if (cmd.equals("acao-cadastrar-cliente")) {
				System.out.println("GravarClienteCommand: irei cadastrar cliente.");
				if (login.existe(cliente.getUserid())) {
					erros.add("Cliente " + cliente.getUserid() + " j existe!");
				} else {
					dadosValidos = isValid(cliente, erros, login);
				}
				if (dadosValidos) {
					login.cadastrar(cliente);
					session.setAttribute("cliente", cliente); // cria sessao do usuario
				}
			} else if (cmd.equals("acao-editar-cliente")) {
				System.out.println("GravarClienteCommand: irei atualizar dados do cliente.");
				ClienteVO clienteAntigo = (ClienteVO)session.getAttribute("cliente");
				if (clienteAntigo == null) {
					erros.add("Sesso inexistente ou expirada. Faa login!");
				} else if (!clienteAntigo.getUserid().equals(cliente.getUserid())) {
					erros.add("Erro: tentativa de alterar userid!");
				} else {
					dadosValidos = isValid(cliente, erros, login);
				}
				if (dadosValidos) {
					login.atualizar(cliente);
					session.setAttribute("cliente", cliente); // altera dados de sessao do usuario
				}
			}
		} catch (LojaException e) {
			throw new CommandException("GravarClienteCommand: " + e);
		}
		
		if (!dadosValidos) {
			request.setAttribute("erros", erros);
			return failureUrl;
		}
		
		request.setAttribute("cliente", cliente); // para uso dos tags
		return successUrl;
    }
	
	private boolean isValid(ClienteVO cliente, List erros, LoginBusinessDelegate login) {
		System.out.println("GravarClienteCommand: Validando cliente...");
		String userid = cliente.getUserid();
		String nome = cliente.getNome();
		String endereco = cliente.getEndereco();
		String email = cliente.getEmail();
		String senha = cliente.getSenha();
		String cartao = cliente.getCartao();
		
		if (userid.equals("")) {
			erros.add("User id no pode ser vazio!");
			return false;
		}
		if (nome.equals("")) {
			erros.add("Nome no pode ser vazio!");
		}
		if (endereco.equals("")) {
			erros.add("Endereo no pode ser vazio!");
		}
		if (email.equals("")) {
			erros.add("E-mail no pode ser vazio!");
		}
		if (email.indexOf("@") == -1) {
			erros.add("Formato incorreto para e-mail!");
		}
		if (senha.equals("")) {
			erros.add("Senha no pode ser vazia!");
		}
		if (senha.length() < 6) {
			erros.add("Senha deve ter no mnimo 6 caracteres!");
		}
		if (senha.indexOf(userid) != -1 || 
		    senha.indexOf(nome) != -1) {
			erros.add("Senha no pode conter nome ou userid!");
		}
		if (cartao.equals("")) {
			erros.add("Cartao no pode ser vazio!");
		}
		if (erros.size() > 0) {
			return false;
		}
		return true;
	}

    /**
     * Object.toString() implementation.
     */
    public String toString() {
        return this.getClass().getName() + "(\"" + successUrl + "\", \"" + failureUrl + "\")";
    }
    
    /**
     * Object.equals() implementation.
     */
    public boolean equals(Object obj) {
        if ( !(obj instanceof GravarClienteCommand) ) return false;
        GravarClienteCommand cmd = (GravarClienteCommand) obj;
        return successUrl.equals(cmd.successUrl) && failureUrl.equals(cmd.failureUrl);
    }
    
    /**
     * Object.hashCode() implementation.
     */
    public int hashCode() {
        return 37 * successUrl.hashCode() * failureUrl.hashCode();
    }
}
