package loja.web.command;
 
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;

import loja.*;
import loja.security.Authenticator;
import loja.web.*;

/**
 * Simple redirection.
 * $Revision: 1.4 $ $Date: 2003/05/27 03:37:59 $ (helder@acm.org).
 */
public class LoginCommand implements Command {
    
    private String successUrl;
    private String failureUrl;
    
    public LoginCommand(String successUrl, String failureUrl) {
        this.successUrl = successUrl;
        this.failureUrl = failureUrl;
    }
    
    public Object execute(HttpServletRequest request, HttpServletResponse response, BusinessDelegate delegate) throws CommandException {

		LoginBusinessDelegate login = (LoginBusinessDelegate)delegate;
		
        String userid = request.getParameter("userid");
		String senha = request.getParameter("senha");
		String linkCadastro = request.getParameter("link-cadastro");
		
		List erros = new ArrayList();
		
		if (userid == null || senha == null) {
			// Verifica se usuario ja estah logado na sessao
			ClienteVO cliente = (ClienteVO)request.getSession().getAttribute("cliente");
			if (cliente == null) {
				return failureUrl;
			} else {
				userid = cliente.getUserid();
				senha  = cliente.getSenha();
			}
		}
		
		try {
			Authenticator.login(userid.trim(), senha.trim());
		} catch (javax.security.auth.login.LoginException e) {
			erros.add("Usuario ou senha invalidos. No  usurio? <a href='"+linkCadastro+"'>Cadastre-se</a>!");
			request.setAttribute("erros", erros);
			return failureUrl;
		}
		
		try {
			ClienteVO cliente = login.mostrar(userid);
			request.setAttribute("cliente", cliente);
			HttpSession session = request.getSession();
			session.setAttribute("cliente", cliente); // cria sessao do usuario
		} catch (LojaException e) {
			throw new CommandException("Login failed: " + e);
		}
		return successUrl;
    }
	
    /**
     * Object.toString() implementation.
     */
    public String toString() {
        return this.getClass().getName() + "(\"" + successUrl + "\", \"" + failureUrl + "\")";
    }
    
    /**
     * Object.equals() implementation.
     */
    public boolean equals(Object obj) {
        if ( !(obj instanceof LoginCommand) ) return false;
        LoginCommand cmd = (LoginCommand) obj;
        return successUrl.equals(cmd.successUrl) && failureUrl.equals(cmd.failureUrl);
    }
    
    /**
     * Object.hashCode() implementation.
     */
    public int hashCode() {
        return 37 * successUrl.hashCode() * failureUrl.hashCode();
    }
}
