package loja.web.command;
 
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;

import loja.*;
import loja.web.*;

/**
 * Simple redirection.
 * $Revision: 1.3 $ $Date: 2003/05/26 09:17:05 $ (helder@acm.org).
 */
public class ProcessarCompraCommand implements Command {
    
    private String successUrl;
    private String failureUrl;
    
    public ProcessarCompraCommand(String successUrl, String failureUrl) {
        this.successUrl = successUrl;
        this.failureUrl = failureUrl;
    }
    
    public Object execute(HttpServletRequest request, HttpServletResponse response, BusinessDelegate delegate) throws CommandException {

		LojaBusinessDelegate loja = (LojaBusinessDelegate)delegate;
		
		ClienteVO cliente = (ClienteVO)request.getSession().getAttribute("cliente");
		List erros = new ArrayList();
		
		if (cliente == null) {
			erros.add("Cliente nao identificado! Faa login antes!");
			request.setAttribute("erros", erros);
			return failureUrl;
		}
		
		try {
			ItemVO[] carrinho = loja.listarConteudoCarrinho();
			CompraVO compra = loja.processarCompra(cliente.getUserid(), carrinho);
			
			String mensagens = verificarEstoque(carrinho, compra.getItens());
			
			request.setAttribute("compra", compra);
			request.setAttribute("itens", compra.getItens());
			request.setAttribute("cliente", cliente);
			request.setAttribute("info", mensagens);
			
		} catch(LojaException e) {
			throw new CommandException(e);
		}
		return successUrl;
    }
	
	private String verificarEstoque(ItemVO[] carrinho, ItemVO[] itensCompra) {
		StringBuffer buffer = new StringBuffer();
		String pre = "<tr><td>";
		String suf = "</td></tr>";
		
		Map carrinhoMap = getMap(carrinho);
		Map itensCompraMap = getMap(itensCompra);
		
		Iterator it = carrinhoMap.keySet().iterator();
		while(it.hasNext()) {
			String codigo = (String)it.next();
			if (itensCompraMap.containsKey(codigo)) {
				int carQte = ((Integer)carrinhoMap.get(codigo)).intValue();
				int iteQte = ((Integer)itensCompraMap.get(codigo)).intValue();
				if (iteQte < carQte) { // houve reduo devido a estoque!
					String msg = "No havia estoque suficiente para atender ao seu pedido de "+carQte+" unidades do produto [" + codigo + "], cujo estoque s continha "+iteQte+" unidades. " +
					             "O seu pedido foi recalculado para incluir os itens disponveis em estoque.";
					buffer.append(pre).append(msg).append(suf);
				}
			} else { // item est esgotado
				String msg = "O item " + codigo + " est esgotado e no foi includo no seu pedido.";
				buffer.append(pre).append(msg).append(suf);
			}
		}
		return buffer.toString();
	}
	
	private Map getMap(ItemVO[] itens) {
		Map itensMap = new HashMap(itens.length);
		for (int i = 0; i < itens.length; i++) {
			String codigo = itens[i].getProdutoId();
			Integer qte   = new Integer(itens[i].getQuantidade());
			itensMap.put(codigo, qte);
		}
		return itensMap;
	}

    /**
     * Object.toString() implementation.
     */
    public String toString() {
        return this.getClass().getName() + "(\"" + successUrl + "\", \"" + failureUrl + "\")";
    }
    
    /**
     * Object.equals() implementation.
     */
    public boolean equals(Object obj) {
        if ( !(obj instanceof ProcessarCompraCommand) ) return false;
        ProcessarCompraCommand cmd = (ProcessarCompraCommand) obj;
        return successUrl.equals(cmd.successUrl) && failureUrl.equals(cmd.failureUrl);
    }
    
    /**
     * Object.hashCode() implementation.
     */
    public int hashCode() {
        return 37 * successUrl.hashCode() * failureUrl.hashCode();
    }
}
