package loja.web.command;
 
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;

import loja.*;
import loja.web.*;

/**
 * Simple redirection.
 * $Revision: 1.3 $ $Date: 2003/05/26 09:17:05 $ (helder@acm.org).
 */
public class RealizarCompraCommand implements Command {
    
    private String successUrl;
    private String failureUrl;
    
    public RealizarCompraCommand(String successUrl, String failureUrl) {
        this.successUrl = successUrl;
        this.failureUrl = failureUrl;
    }
    
    public Object execute(HttpServletRequest request, HttpServletResponse response, BusinessDelegate delegate) throws CommandException {

		LojaBusinessDelegate loja = (LojaBusinessDelegate)delegate;
		
		String compraId = request.getParameter("compra");
		CompraVO compra = new CompraVO();
		compra.setNumero(compraId);
		
		try {
			loja.comprar();
			loja.esvaziarCarrinho();
			request.setAttribute("compra", compra);
		} catch (LojaException e) {
			throw new CommandException (e);
		}

		return successUrl;
    }

    /**
     * Object.toString() implementation.
     */
    public String toString() {
        return this.getClass().getName() + "(\"" + successUrl + "\", \"" + failureUrl + "\")";
    }
    
    /**
     * Object.equals() implementation.
     */
    public boolean equals(Object obj) {
        if ( !(obj instanceof RealizarCompraCommand) ) return false;
        RealizarCompraCommand cmd = (RealizarCompraCommand) obj;
        return successUrl.equals(cmd.successUrl) && failureUrl.equals(cmd.failureUrl);
    }
    
    /**
     * Object.hashCode() implementation.
     */
    public int hashCode() {
        return 37 * successUrl.hashCode() * failureUrl.hashCode();
    }
}
