package loja.web.command;
 
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;

import loja.*;
import loja.web.*;

/**
 * $Revision: 1.2 $ $Date: 2003/05/24 02:02:08 $ (helder@acm.org).
 */
public class SolicitarCompraCommand implements Command {
    
    private String successUrl;
    private String failureUrl;
    
    public SolicitarCompraCommand(String successUrl, String failureUrl) {
        this.successUrl = successUrl;
        this.failureUrl = failureUrl;
    }
    
    public Object execute(HttpServletRequest request, HttpServletResponse response, BusinessDelegate delegate) throws CommandException {

		AdminLojaBusinessDelegate loja = (AdminLojaBusinessDelegate)delegate;
		String codigo = request.getParameter("codigo");
		String quantidade = request.getParameter("quantidade");
		int qte = 0;
		List erros = new ArrayList();
		if (codigo == null || codigo.trim().length() == 0) {
			erros.add("Cdigo vazio! Selecione um codigo de produto existente.");
		}
		if (quantidade != null && quantidade.trim().length() != 0) {
			try {
				qte = Integer.parseInt(quantidade.trim());
			} catch (NumberFormatException e) {}
			if(qte == 0) {
				erros.add("Quantidade invlida! Escolha um nmero maior que zero.");
			}
		} else {
			erros.add("Quantidade invlida! Escolha um <b>nmero</b> maior que zero.");
		}
		if (erros.size() > 0) {
			request.setAttribute("erros", erros);
			return failureUrl;
		}
		
		try {
			loja.solicitarCompra(codigo.trim(), qte);
		} catch (LojaException e) {
			throw new CommandException(e);
		}
		return successUrl;
    }
    
    /**
     * Object.toString() implementation.
     */
    public String toString() {
        return this.getClass().getName() + "(\"" + successUrl + "\", \"" + failureUrl + "\")";
    }
    
    /**
     * Object.equals() implementation.
     */
    public boolean equals(Object obj) {
        if ( !(obj instanceof SolicitarCompraCommand) ) return false;
        SolicitarCompraCommand cmd = (SolicitarCompraCommand) obj;
        return successUrl.equals(cmd.successUrl) && failureUrl.equals(cmd.failureUrl);
    }
    
    /**
     * Object.hashCode() implementation.
     */
    public int hashCode() {
        return 37 * successUrl.hashCode() * failureUrl.hashCode();
    }
}
