package loja.web.taglib;
 
import javax.servlet.*;
import javax.servlet.jsp.*; // JspWriter, JspException, JspTagException
import javax.servlet.jsp.tagext.*; // TagSupport, Tag

import loja.*;

/**
 * $Revision: 1.3 $ $Date: 2003/05/26 08:22:27 $ (helder@acm.org).
 */
public class ElementoTag extends TagSupport {
    
    private String  property, 
                    beanName;
    
    /**
     * Setter for the name attribute.
     */
    public void setName(String beanName) {
        this.beanName = beanName;
    }
    
    /**
     * Setter for the property attribute.
     */
    public void setProperty(String property) {
        this.property = property;
    }

    
    /**
     * Processed before the contents of the tag. This tag can be used with or without a
     * <listarElementos> parent. If it is used without one, it will require a beanName
     * attribute matching an available bean in request scope. Otherwise, it will look
     * for an ancestor of ListarElementosTag class and extract its current message.
     */
    public int doStartTag() throws JspException {
        // Gets reference to this page's out scripting variable
        JspWriter out = pageContext.getOut();
        try {
            // get bean from request (available when used without parent)
            Object bean = null;
            if (beanName != null) {
                bean = pageContext.getRequest().getAttribute(beanName);
            }
            if (bean == null) { // either beanName is null or beanName does not correspond to request attribute
                // get bean name from page context inside <printMessages> tag
                ListarElementosTag parent = (ListarElementosTag)
                    TagSupport.findAncestorWithClass(this, ListarElementosTag.class);
                if (parent == null) {
                    throw new JspException("Bean is null or missing required <listarElementos> tag when name attribute is omitted.");
                }
                // Get object from current iteration
                bean = parent.getObject();
                // if it is still null, then throw an exception
                if (bean == null) {
                    throw new JspException("Elemento is not defined in tag context for JSP page.");
                }
            }
            // TagUtils is a local utility class
            String field = TagUtils.getPropertyValue(bean, property);
            if (field == null) {
                throw new JspException(property + " is not defined in " + bean.getClass() + ".");
            }
            out.print(field);
        } catch (java.io.IOException e) {
            throw new JspTagException("ElementoTag: " + e);
        }
        return Tag.SKIP_BODY;
    }

    public void release() {
        super.release();
        property = null;
        beanName = null;
    }
    
}
