package loja.web.taglib;
 
import javax.servlet.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

import loja.ProdutoVO;

/**
 * This tag is used to iterate over several <elemento> tags. It uses a previously
 * defined array that must exist in request. It's array attribute must contain the
 * name (request attribute name) of this array of Elemento.
 * $Revision: 1.1 $ $Date: 2003/05/25 15:53:06 $ (helder@acm.org).
 */
public class ListarElementosTag extends BodyTagSupport {
    
    private String beanArrayName;
    private Object[] beans;
    private Object currentObject;
    private int position = 0;

    /**
     * Getter to pass produto beans to child tags. Package-private.
     */
    Object getObject() {
        return currentObject;
    }
    
    /**
     * Setter for the array attribute.
     */
    public void setArray(String beanArrayName) {
        this.beanArrayName = beanArrayName;
    }
    
    /**
     * Done once when the tag starts.
     */
    public int doStartTag() throws JspException {
        // Get bean array that should have been set by listarProdutosCommand
        Object obj = pageContext.getRequest().getAttribute(beanArrayName);
        if (obj != null) {
            // must be array
            if (obj instanceof Object[]) {
                beans = (Object[])obj;
                if (beans.length > 0) {
                    // set bean as attribute so nested tags can use it
                    currentObject = beans[0]; // first bean
                    position++;
                    return EVAL_BODY_BUFFERED; // continue processing contents of this tag
                } else {
                    position = 0;
                    return SKIP_BODY;
                }
            } else {
                throw  new JspTagException(beanArrayName + " is not an array of Object!");  
            }
        } else {
            throw new JspTagException("No object available for " + beanArrayName + " request attribute.");
        }
    }
     
    /**
     * Done when the tag is closing. In this case, this method is called
     * recursively until the array is done.
     */
    public int doAfterBody() throws JspException {
        // Get contents of this tag
        BodyContent body = this.getBodyContent();
        try {
            // get buffer and flush it
            body.writeOut( this.getPreviousOut() );
        } catch (java.io.IOException e) {
            throw new JspTagException("Unexpected I/O exception: " + e);
        }
        // clear the contents of the tag (we already flushed what we wanted)
        body.clearBody();
        // Keep processing the body of this tag until the array is done
        if (position < beans.length) {
            // set bean as attribute for nested tags
            currentObject = beans[position];
            position++;
            return EVAL_BODY_BUFFERED;
        } else {
            position = 0;
            return SKIP_BODY;
        }
    }

    public void release() {
        super.release();
        beanArrayName = null;
        beans = null;
        position = 0;
    }
}
