package loja.ejb;

import javax.ejb.*;
import java.util.*;
import java.math.BigDecimal;

import loja.integration.*;
import loja.*;

public class ProdutoBean implements EntityBean {
	
	protected EntityContext ctx;
	
	private String codigo;
	private String nome;
	private BigDecimal preco;
	private int quantidade;
	
	ProdutoDAO dao; 
	
	//////////////////// life-cycle ////////////////////////////
	
	/////////// 1. HOME ///////////////

	// create e postCreate
	public void ejbPostCreate(String codigo) {}
	public ProdutoPK ejbCreate(String codigo) throws CreateException {
		return this.ejbCreate(codigo, "", new BigDecimal(0.0), 0);
	}
	
	public void ejbPostCreate(String codigo, String nome, BigDecimal preco) {}
	public ProdutoPK ejbCreate(String codigo, String nome, BigDecimal preco) throws CreateException {
		return this.ejbCreate(codigo, nome, preco, 0);
	}
	
	public void ejbPostCreate(String codigo, String nome, BigDecimal preco, int quantidade) {}	
	public ProdutoPK ejbCreate(String codigo, String nome, BigDecimal preco, int quantidade) throws CreateException {
		System.out.println("ProdutoBean: ejbCreate("+codigo+","+nome+","+preco+","+quantidade+")");
		try {
			ProdutoVO vo = new ProdutoVO(codigo, nome, preco, quantidade);
			dao.create(vo);
			this.codigo = codigo;
			this.nome = nome;
			this.preco = preco;
			this.quantidade = quantidade;
			return new ProdutoPK(codigo);
		} catch (LojaException e) {
			throw new CreateException("ProdutoBean: ejbCreate() - " + e);
		}
	}	
	
	// finders
	public ProdutoPK ejbFindByPrimaryKey(ProdutoPK pk) throws FinderException {
		System.out.println("ProdutoBean: ejbFindByPrimaryKey("+pk+")");
		try {
			if(dao.exists(pk.codigo)) {
				return pk;
			} else throw new FinderException("Produto " + pk + " nao existe!");
		} catch (LojaException e) {
			throw new FinderException("ProdutoBean: ejbFindByPrimaryKey() - " + e);
		}
	}
	
	// findAll
	public Collection ejbFindAll() throws FinderException {
		System.out.println("ProdutoBean: findAll()");
		try {
			String[] codigos = dao.getAll();
			List codigosPK = new ArrayList();
			for (int i = 0; i < codigos.length; i++) {
				codigosPK.add( new ProdutoPK(codigos[i]) );
			}
			return codigosPK;
		} catch (LojaException e) {
			throw new FinderException("ProdutoBean: findAll() - " + e);
		}
	}
	
	
	/////////// 2. CONTAINER CALL-BACK ///////////////////
	// load
	public void ejbLoad() {
		System.out.println("ProdutoBean: ejbLoad()");
		try {
			ProdutoPK pk = (ProdutoPK)ctx.getPrimaryKey();
			
			ProdutoVO vo = dao.load(pk.codigo);
			this.codigo = vo.getCodigo();
			this.nome = vo.getNome();
			this.preco = vo.getPreco();
			this.quantidade = vo.getQuantidade();
		} catch (LojaException e) {
			throw new EJBException("ProdutoBean: ejbLoad() - " + e);
		}
	}
	
	// store
	public void ejbStore() {
		System.out.println("ProdutoBean: ejbStore()");
		// Montar o VO com dados do objeto
		try {
			ProdutoVO vo = new ProdutoVO(this.codigo, this.nome, this.preco, this.quantidade);
			dao.store(vo);
		} catch (LojaException e) {
			throw new EJBException("ProdutoBean: ejbStore() - " + e);
		}
	}
	
	// remove
	public void ejbRemove() {
		System.out.println("ProdutoBean: ejbRemove()");
		try {
			ProdutoPK pk = (ProdutoPK)ctx.getPrimaryKey();
			dao.remove(pk.codigo);
		} catch (LojaException e) {
			throw new EJBException("ProdutoBean: ejbRemove() - " + e);
		}
	}
	
	public void setEntityContext(EntityContext ctx) {
		System.out.println("ProdutoBean: setEntityContext()");
		this.ctx = ctx;
		try {
			dao = JdbcProdutoDAO.getInstance(); 
		} catch (LojaException e) {
			throw new EJBException("ProdutoBean: setEntityContext(): Falha na inicializacao do DAO: " + e);
		}
	}
	
	public void unsetEntityContext() {
		ctx = null;
	}
	
	public void ejbActivate() {}
	public void ejbPassivate() {}
	
	
	//////////// BUSINESS AND ACESSOR ///////////////
	
	public String getCodigo() {
		return codigo;
	}
	public void setCodigo(String codigo) {
		this.codigo = codigo;
	}
	
	public void setNome(String nome) {
		this.nome = nome;
	}
	public String getNome() {
		return nome;
	}
	
	public void setPreco(BigDecimal preco) {
		this.preco = preco;
	}
	public BigDecimal getPreco() {
		return preco;
	}
	
	public int getQuantidade() {
		return quantidade;
	}
	public void setQuantidade(int quantidade) {
		this.quantidade = quantidade;
	}
	
	public void addItens(int quantidade) {
		System.out.println("ProdutoBean: addItens("+quantidade+")");
		this.quantidade += quantidade;
	}

}
