package loja.ejb;

import loja.integration.*;
import loja.*;

import javax.ejb.*;
import javax.jms.*;
import java.math.BigDecimal;
import java.util.*;

public class AdminLojaSessionFacadeBean implements SessionBean {
	
	private ProdutoLocalHome produtoHome;
	private SessionContext ctx;
	
	public void ejbCreate() throws CreateException {
		System.out.println("AdminLojaSessionFacadeBean: ejbCreate()");
		try {
			ServiceLocator locator = ServiceLocator.getInstance();
			produtoHome = (ProdutoLocalHome) locator.findLocalEJB("ejb/loja-simples/produto");
		} catch (LojaException e) {
			throw new CreateException("Service Locator failure: " + e);
		}
	}
	
	public String criarProduto(String nome, BigDecimal preco) throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: criarProduto("+ nome+", "+preco+")");
		ProdutoDAO dao = JdbcProdutoDAO.getInstance();
		String codigo = dao.criarCodigo();
		try {
			System.out.println("AdminLojaSessionFacadeBean: Criando produto codigo " + codigo);
			produtoHome.create(codigo, nome, preco, 0);
		} catch (CreateException e) {
			throw new LojaException("Incapaz de criar produto " + nome);
		}
		return codigo;
	}
	
	public void removerProduto(String codigo) throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: removerProduto("+ codigo+")");
		try {
			ProdutoLocal p = produtoHome.findByPrimaryKey(new ProdutoPK(codigo));
			if (p.getQuantidade() == 0) {
				p.remove();
			} else {
				throw new LojaException("Esvazie o estoque de produtos " + codigo + " antes de remover do banco.");
			}
		} catch (FinderException e) {
			throw new LojaException(e);
		} catch (RemoveException e) {
			throw new LojaException(e);
		}
	}
	public ProdutoVO[] listarProdutos() throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: listarProdutos()");
		try {
			Collection beans = produtoHome.findAll();
			System.out.println("AdminLojaSessionFacadeBean: listarProdutos():Achados " + beans.size() + " beans.");
			Iterator it = beans.iterator();
			List produtos = new ArrayList();
			while(it.hasNext()) {
				ProdutoLocal produtoBean = (ProdutoLocal) it.next();
				ProdutoVO produtoVO = new ProdutoVO();
				produtoVO.setCodigo(produtoBean.getCodigo());
				produtoVO.setNome(produtoBean.getNome());
				produtoVO.setPreco(produtoBean.getPreco());
				produtoVO.setQuantidade(produtoBean.getQuantidade());
				produtos.add(produtoVO);
			}
			ProdutoVO[] buffer = new ProdutoVO[produtos.size()];
			return (ProdutoVO[]) produtos.toArray(buffer);
		} catch (FinderException e) {
			throw new LojaException(e);
		}
	}
	
	public void solicitarCompra(String codigo, int quantidade) throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: solicitarCompra("+ codigo+", "+quantidade+")");
		try {
			ServiceLocator locator = ServiceLocator.getInstance();
			Queue queue = locator.findQueue("queue/testQueue");
			QueueConnection con = locator.findDefaultJMSQueueConnection();
			
			QueueSession session = con.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
			String text = codigo +"|"+ quantidade;
			TextMessage message = session.createTextMessage(text);
			QueueSender sender = session.createSender(queue);
			sender.send(message);
		} catch (JMSException e) {
			throw new LojaException(e);
		}
	}
	
	public void esvaziarEstoque(String codigo) throws LojaException {
		System.out.println("AdminLojaSessionFacadeBean: esvaziarEstoque("+ codigo+")");
		try {
			ProdutoLocal p = produtoHome.findByPrimaryKey(new ProdutoPK(codigo));
			if (p.getQuantidade() != 0) {
				p.setQuantidade(0);
			}
		} catch (FinderException e) {
			throw new LojaException(e);
		}
	}
	
	public void ejbActivate() {}
	public void ejbPassivate() {}
	public void ejbRemove() {}
	public void setSessionContext(SessionContext ctx) {
		this.ctx = ctx;
	}
}
