package loja.web; 

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.InputStream;

import loja.*;

/**
 * This is the front controller of the application.
 * $Revision: 1.3 $ $Date: 2002/10/06 14:24:58 $ (helder@acm.org).
 */
public class AdminServlet extends HttpServlet {

    private AdminLojaBusinessDelegate loja;

    private CommandFactory commandFactory;

    public void init() throws UnavailableException {
        String mappingsFile = getInitParameter("mappings-file");
        if (mappingsFile == null) {
            throw new UnavailableException("Required parameter mappings-file not found in deployment descriptor");
        }
		try {
			loja = new AdminLojaBusinessDelegate();
		} catch (LojaException e) {
			throw new UnavailableException("Falha na inicializao do Business Delegate: "+e);
		}
        
        // This class receives the name of an XML file which contains command-action object mappings
        // and returns an object that can return action objects by name.
        try {
            InputStream stream = getServletContext().getResourceAsStream(mappingsFile);
            commandFactory = CommandFactory.getInstance(stream);
        } catch (Exception e) {
            throw new UnavailableException("Falha na inicializao: "+e);
        }
    }

    public void service(HttpServletRequest request,
                        HttpServletResponse response) throws ServletException, java.io.IOException {

        Command command = null;
        String commandName = request.getParameter("cmd");
        if (commandName == null) {
            command = commandFactory.getCommand("default");
        } else {
            try {
                command = commandFactory.getCommand(commandName);
            } catch (CommandNotFoundException e) {
                command = commandFactory.getCommand("default");
            }
        }

        Object result = command.execute(request, response, loja);
        if (result instanceof String) {
            RequestDispatcher dispatcher = request.getRequestDispatcher((String)result);
            dispatcher.forward(request, response);
        } else {
            throw new ServletException("Command returned an invalid response: " + result);
        }
    }
}
