package loja.web;

import java.util.HashMap;
import org.xml.sax.*;
import java.lang.reflect.Constructor;

/**
 * This class uses the SAX API to parse the XML file which contains the action mappings.
 * $Revision: 1.2 $ $Date: 2002/09/28 20:27:30 $ (helder@acm.org).
 */
public class CommandFactorySAXHandler extends org.xml.sax.helpers.DefaultHandler {
	
	private HashMap commands;
    private CommandData currentCommandData; // JavaBean which encapsulates command data
    private String currentText;
	private boolean inCommand = false;
	
	public CommandFactorySAXHandler(HashMap commands) {
		this.commands = commands;
	}
    
    /**
     * Called when a start tag is found.
     */
    public void startElement(String ns, String ln, String name, Attributes atts) {
        if (name.equals("command")) {
            inCommand = true;
            currentCommandData = new CommandData();
        }
    }
    
    /**
     * Called when an end tag is found.
     */
    public void endElement(String ns, String ln, String name) {
        if (name.equals("command")) {
            inCommand = false;
            try {
                Class commandClass = Class.forName(currentCommandData.getClassName());
                Constructor constructor = commandClass.getConstructor(new Class[] {String.class, String.class});
                String[] args = { currentCommandData.getSuccessParam(), currentCommandData.getFailureParam() };
                Command command = (Command)constructor.newInstance(args);
                
                commands.put( currentCommandData.getCommandName(), command);
            } catch (Exception e) {
                if (e instanceof RuntimeException) {
                    throw (RuntimeException) e;
                }
                System.out.println("Reflection Error: " + e);
            }
            currentCommandData = null;
        }
		
        if (inCommand) {
            if (name.equals("name")) {
                currentCommandData.setCommandName(currentText);
            } else if (name.equals("class")) {
                currentCommandData.setClassName(currentText);
            } else if (name.equals("success-url")) {
                currentCommandData.setSuccessParam(currentText);
            } else if (name.equals("failure-url")) {
                currentCommandData.setFailureParam(currentText);
            }
            currentText = null;
        }
    }
    
    /**
     * Called when a character node is found.
     */
    public void characters(char[] chars, int start, int length) {
        if (inCommand) {
            currentText = (new String(chars, start, length)).trim();
        }
    }
    
    /////////////////// End SAX /////////////////////////////////
}
